/*
 * Sungka v0.10.
 * AbstractSungkaPlayerView.java - Player View base class
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import org.omg.CORBA.ORB;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.IntHolder;
import org.omg.CORBA.BAD_PARAM;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POA;

import java.util.*;
import java.text.MessageFormat;

/**
 * AbstractSungkaPlayerView partially implements the player side of the
 * game. All views of the game that derive from this class and override a method
 * must call the superclass's methods at some point in processing.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public abstract class AbstractSungkaPlayerView implements SungkaPlayerView, SungkaConstants {

	/** The SungkaPlayer model this view is associated with */
	protected SungkaPlayerServant myModel;

	/**
	 * Constructs a new SungkaPlayerView. All overriding classes must call
	 * this constructor.
	 * @param model The SungkaPlayer model to associate this view with.
	 */
	protected AbstractSungkaPlayerView(SungkaPlayerServant model) {
		myModel = model;
		model.setView(this);
	}

	/**
	 * Formats a string in the format specified by Java MessageFormat.
	 * @param format The string format
	 * @param arg The string to replace all {0}'s in the format with.
	 * @return The formatted string
	 */
	protected static String formatString(String format, String arg) {
		return MessageFormat.format(format, new Object[]{ arg });
	}

	/**
	 * Processes an update board message. Called by model after the entire
	 * board contents have been changed.
	 */
	public void processUpdateBoard() {
	}

	/**
	 * Processes an update parameters message. Called by the model when game
	 * parameters (e.g. number of participants, number of cups) have changed.
	 */
	public void processUpdateParams() {
	}

	/**
	 * Processes a cup update message
	 * @param player The player which had a cup value changed
	 * @param cup The cup number which changed value
	 */
	public void processUpdateCup(int player, int cup) {
	}

	/**
	 * Notifies view to reset game state when connecting to a new server.
	 * The model calls this after the model's state has been reset. A view
	 * must do any necessary processing to come as close to program startup
	 * state as possible.
	 */
	public void gameReset() {
	}

	/**
	 * Processes playerAdded message.
	 * @param s A SungkaPlayerInfo object with all the information of the added player.
	 */
	public void processPlayerAdded(SungkaPlayerInfo s) {
		log(formatString(resources.getString("newplayer"), s.getName()), LOG_PLAYER);
	}

	/**
	 * Processes playerRemoved message.
	 * @param s A SungkaPlayerInfo object with all the information of the removed player.
	 */
	public void processPlayerRemoved(SungkaPlayerInfo s) {
		log(formatString(resources.getString("leftgame"), s.getName()), LOG_PLAYER);
	}

	/**
	 * Processes endTurn message.
	 * @param player The player number of the player ending his turn.
	 */
	public void processEndTurn(int player) {
		log(formatString(resources.getString("endturn"), myModel.getPlayerName(player)), LOG_MOVE);
	}

	/**
	 * Processes playerMoved message.
	 * @param player The player number of the player making his move.
	 * @param cup The cup number the player is making his move from.
	 */
	public void processPlayerMoved(int player, int cup) {
		log(MessageFormat.format(resources.getString("movedin"), new Object[]{ myModel.getPlayerName(player), new Integer(cup) }), LOG_MOVE);
	}

	/**
	 * Processes gameOver message.
	 * @param winner The order number of the winner of this game. Negative if error occured,
	 *               equal to the number of players if there is a tie.
	 */
	public void processGameOver(int winner) {
		if (winner == SungkaPlayer.GAMEOVER_TIE) {
			log(resources.getString("tiegame"), LOG_MOVE);
		} else if (winner < 0) {
			log(resources.getString("nowinner"), LOG_MOVE);
		} else if (winner == myModel.getMyNumber()) {
			log(resources.getString("youwin"), LOG_MOVE);
		} else {
			log(formatString(resources.getString("youlose"), myModel.getPlayerName(winner)), LOG_MOVE);
		}
	}

	/**
	 * Processes chatMessage message.
	 * @param msg The chat message.
	 * @param player The player the message was received from.
	 */
	public void processChatMessage(String msg, int player) {
		log(MessageFormat.format(resources.getString("chatmsg"), new Object[]{ myModel.getPlayerName(player), msg }), LOG_CHAT);
	}

	/** 
	 * Processes disconnect message. Called by the model after it has already
	 * disconnected from the server.
	 */
	public void processDisconnect() {
		log(resources.getString("disconnected"), LOG_ERROR);
	}

	/** Processes end of round message. */
	public void processEndRound() {
	}
}
