/*
 * Sungka v0.10.
 * GUISungkaPlayerView.java - Player GUI View class
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.table.*;

import java.text.MessageFormat;
import org.omg.CORBA.ORB;

/**
 * GUISungkaPlayerView implements a GUI for the player side of the game.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class GUISungkaPlayerView extends AbstractSungkaPlayerView implements Runnable {
	/** The actual GUI JFrame shown to the user. */
	SungkaPlayerGUI myGUI;

	/** Reference to the connect dialog object that the user last opened. */
	SungkaConnectDialog retDialog = null;

	/** Controls when to stop the GUI and close program. */
	volatile boolean doShutdown = false;

	/** Reference to the game server that this game starts */
	SungkaRegisterServant server = null;

	/**
	 * Constructs a new GUI view of the Sungka game.
	 * @param servant The servant that this view will be associated with.
	 */
	GUISungkaPlayerView(SungkaPlayerServant servant) {
		super(servant);
		myGUI = new SungkaPlayerGUI();
	}

	/**
	 * Logs a message to show to the user.
	 * @param s The actual message
	 * @param msgType A string representing the type of message.
	 *                Used to format the string output.
	 */
	public void log(final String s, final String msgType) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Document doc = myGUI.gameLog.getDocument();
				try {
					doc.insertString(doc.getLength(), s + "\n", myGUI.gameLog.getStyle(msgType));
				} catch (BadLocationException e) {
					System.err.println("Could not insert text!");
					e.printStackTrace();
				}

				// Scroll pane to bottom to make new message visible
				myGUI.gameLog.scrollRectToVisible(new Rectangle(0,myGUI.gameLog.getHeight(),1,1));
				if (msgType.equals(LOG_MOVE) || msgType.equals(LOG_ERROR)) {
					myGUI.statusLabel.setText(s);
				}
			} // run
		});
	} // log

	/**
	 * Processes an update parameters message. Called by the model when game
	 * parameters (e.g. number of participants, number of cups) have changed.
	 */
	public void processUpdateParams() {
		// We use an update params message as a signal of when to initialize
		// the boardPane.
		if (myGUI.boardPane == null || 
				myGUI.boardPane.myCups.length != myModel.getNumParticipants() ||
				myGUI.boardPane.myCups[0].length != myModel.getNumParticipants()) {
			myGUI.initBoard(myModel.isObserver());
		}
		myGUI.chatAction.setEnabled(true);
	}

	/**
	 * Notifies view to reset game state when connecting to a new server.
	 * The model calls this after the model's state has been reset. A view
	 * must do any necessary processing to come as close to program startup
	 * state as possible.
	 */
	public void gameReset() {
		super.gameReset();
		myGUI.gameReset();
	}

	/**
	 * Processes playerAdded message.
	 * @param s A SungkaPlayerInfo object with all the information of the added player.
	 */
	public void processPlayerAdded(SungkaPlayerInfo s) {
		super.processPlayerAdded(s);
		myGUI.playerAdded(s);
	}

	/**
	 * Processes playerRemoved message.
	 * @param s A SungkaPlayerInfo object with all the information of the removed player.
	 */
	public void processPlayerRemoved(SungkaPlayerInfo s) {
		super.processPlayerRemoved(s);
		if (s != null) {
			myGUI.playerRemoved(s);
		}
	}

	/**
	 * Processes startTurn message.
	 * @param player The player number of the player starting his turn.
	 */
	public void processStartTurn(int player) {
		myGUI.startTurn(player);
		if (myModel.myTurn()) {
			log(resources.getString("yourturn"), LOG_MOVE);
		} else {
			log(formatString(resources.getString("waitturn"), myModel.getPlayerName(myModel.getCurrentPlayer())), LOG_MOVE);
		}
	}

	/**
	 * Processes endTurn message.
	 * @param player The player number of the player ending his turn.
	 */
	public void processEndTurn(int player) {
		super.processEndTurn(player);
		myGUI.endTurn(player);
	}

	// No need to override processPlayerMoved

	/**
	 * Processes an update to a cup
	 * @param player The player number of the side receiving the stones.
	 * @param cup The cup number whose amount changed.
	 */
	public void processUpdateCup(int player, int cup) {
		super.processUpdateCup(player, cup);
		int side = (myModel.getPlayerInfo(player)).getOrder();
		myGUI.updateCup(side, cup);
	}

	/**
	 * Processes gameOver message.
	 * @param winner The order number of the winner of this game. Negative if error occured,
	 *               equal to the number of players if there is a tie.
	 */
	public void processGameOver(int winner) {
		super.processGameOver(winner);
		myGUI.readyButton.setVisible(true);
	}

	// No need to override processEndRound, for now
	// No need to override processChatMessage, for now

	/**
	 * Processes an update board message. Called by model after the entire
	 * board contents have been changed.
	 */
	public void processUpdateBoard() {
		super.processUpdateBoard();
		myGUI.boardPane.updateBoard();	
	}

	/** 
	 * Processes disconnect message. Called by the model after it has already
	 * disconnected from the server.
	 */
	public void processDisconnect() {
		int lastPlayer = myModel.getCurrentPlayer();
		super.processDisconnect();
		if (lastPlayer >= 0) {
			// Guards against Null pointer
			SungkaPlayerInfo s = myModel.getPlayerInfo(lastPlayer);
			if (s != null) {
				myGUI.playerModel.endTurn(s);
			}
		}
		myGUI.readyButton.setVisible(false);
		myGUI.chatAction.setEnabled(false);
	}

	/**
	 * Runs this Player view. Useful for starting this view in a new thread.
	 */
	public void run() {
		SungkaConnectDialog dialog = null;
		myGUI.stopServerAction.setEnabled(false);
		while (!doShutdown) {
			myGUI.connectAction.setEnabled(true);
			myGUI.disconnectAction.setEnabled(false);

			synchronized (this) {
				// wait for the dialog to be valid and get settings from it
				while (retDialog == null && !doShutdown) {
					try {
						wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				if (doShutdown) {
					break;
				}
				dialog = retDialog;
				retDialog = null;
			}
			
			myGUI.connectAction.setEnabled(false);
			myGUI.disconnectAction.setEnabled(true);

			try {
				// Connect...
				SungkaRegister sungkaServant = myModel.connectToSungka(dialog.server, dialog.port);
				if (sungkaServant == null) {
					continue;
				}

				// And join.....
				boolean playerJoined = false;
				playerJoined = myModel.joinSungkaGame(sungkaServant, dialog.playerName, dialog.observer);
				sungkaServant._release();
				if (playerJoined) {
					myModel.mainLoop();
				}
			} catch (Exception e) {
				System.err.println("ERROR : " + e) ;
				e.printStackTrace(System.out);
			} // try
		} // while
		// Only get here by doShutdown true.
		myModel.shutdown();
		if (server != null) {
			stopServer(true);
		}
		System.exit(0);
	} // run

	/**
	 * Starts the game server.
	 * @param host The CORBA Naming Service to register with. Null if we are
	 *             just using the Initial Naming Service.
	 * @param port The port of the CORBA Naming Service, or the port to have
	 *             the Initial Naming Service monitor.
	 * @param players The number of participants in the game.
	 * @param cups The number of cups per player.
	 * @param stones The initial number of stones per cup.
	 * @return True if the server started successfully, false otherwise.
	 */
	private boolean startServer(String host, int port, int players, int cups, int stones) {
		try {
			Properties p = new Properties();
			if (host == null) {
				p.setProperty("com.sun.CORBA.POA.ORBPersistentServerPort", port + "");
			} else {
				if (!host.equals("")) {
					p.setProperty("org.omg.CORBA.ORBInitialHost", host);
				}
				if (port != 0) {
					p.setProperty("org.omg.CORBA.ORBInitialPort", port + "");
				}
			}
			ORB orb = ORB.init((String[])null, p);
			server = new SungkaRegisterServant(players, cups, stones);
			((org.omg.CORBA_2_3.ORB)orb).set_delegate(server);

			// Initialize a pipe so that we can capture log data and print it
			// to our server log
			PipedWriter o = new PipedWriter();
			final PrintWriter logPrintWriter = new PrintWriter(o);
			final Reader logReader;
			try {
				logReader = new PipedReader(o);
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}

			// Create the SungkaRegisterView
			server.addView(new SungkaRegisterView() {
				public PrintWriter getLogPrintWriter() {
					return logPrintWriter;
				}

				public void updateState(int state) {
					myGUI.serverLabel.setText(resources.getString("server_state" + state));
				}
			});

			// Register server with name server (either local Initial Name Service, or remote CORBAname service)
			if (!server.registerServer(host != null)) {
				log(resources.getString("server_notregistered"), LOG_ERROR);
				server = null;
				return false;
			}

			new Thread(server).start();
			new Thread(new Runnable() {
				public void run() {
					try {
						// Read won't exit until the server is closed. Once the
						// server closes, we'll exit this and thread will die.
						myGUI.logDialog.logEditorKit.read(logReader,
							myGUI.logDialog.logDocument,
							myGUI.logDialog.logDocument.getEndPosition().getOffset() - 1);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}).start();
			myGUI.startServerAction.setEnabled(false);
			myGUI.stopServerAction.setEnabled(true);
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Stops a running game server.
	 * @param wait True if we want to block until the server has completely
	 *             stopped, false otherwise.
	 */
	private void stopServer(boolean wait) {
		if (server != null) {
			server.shutdown(wait);
			server._orb().shutdown(true);
			server._orb().destroy();
			server = null;
			myGUI.startServerAction.setEnabled(true);
			myGUI.stopServerAction.setEnabled(false);
		}
	}
	
	/**
	 * Runs the GUI Player view from the command line.
	 * @param args Command line arguments to this game. See documentation.
	 */
	public static void main(String [] args) {
		ORB orb = ORB.init(args, null);

		final SungkaPlayerServant playerServant = new SungkaPlayerServant();
		((org.omg.CORBA_2_3.ORB)orb).set_delegate(playerServant);
		final GUISungkaPlayerView myView =
			new GUISungkaPlayerView(playerServant);

		final SungkaPlayerGUI frame = myView.myGUI;
		frame.addWindowListener(new WindowAdapter() {
			private void exit() {
				frame.disconnectAction.actionPerformed(new ActionEvent(frame.getContentPane(),
						ActionEvent.ACTION_PERFORMED, "disconnect"));
				myView.myModel.disconnect();
				synchronized (myView) {
					myView.doShutdown = true;
					myView.notifyAll();
				}
				// run() terminating calls System.exit (uses doShutdown as semaphore)
			}
			// For native close (e.g. X button, Alt-F4)
			public void windowClosing(WindowEvent e) {
				exit();
			}
			// For programatic close (i.e. dispose)
			public void windowClosed(WindowEvent e) {
				exit();
			}
		}); // WindowAdapter

		frame.pack();
		frame.setVisible(true);

		// Start main communication event handling and game playing thread
		new Thread(myView).start();

		// Open connect window on startup. Instead of invoking directly, we queue it up in the AWT EventQueue
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				frame.connectAction.actionPerformed(new ActionEvent(frame.getContentPane(),
						ActionEvent.ACTION_PERFORMED, "connect"));
			}
		});
	} // main

	/**
	 * SungkaPlayerGUI implements the actual user interface visible to the
	 * user.
	 */
	class SungkaPlayerGUI extends JFrame {

		/** The actual pane displaying the Suncka cups. */
		SungkaBoardPane boardPane;
		/** The JList component for the list of players */
		JList playerList;
		/** The list model for the list of players */
		SungkaListModel playerModel;
		/** The game event log */
		JTextPane gameLog;
		/** The view port for the scrolling game event log */
		JViewport gameLogView;
		/** The main menu bar for the game */
		JMenuBar menuBar;
		/** The split pane for the game board/game event log */
		JSplitPane sungkaPane;
		/** The label for the game status. */
		JLabel statusLabel;
		/** The label for the server status */
		JLabel serverLabel;
		/** The "I'm Ready To Play" button */
		final JButton readyButton;
		/** The Dialog box to show/hide for the server log */
		SungkaRegisterLogDialog logDialog = new SungkaRegisterLogDialog(this);

		/** The Action for displaying the connect dialog */
		Action connectAction = new AbstractAction(resources.getString("GUI_sungkamenu_connect")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
			}
			public void actionPerformed(ActionEvent e) {
				SungkaConnectDialog dialog = new SungkaConnectDialog(SungkaPlayerGUI.this);
				dialog.setVisible(true);
				dialog.dispose();

				if (dialog.inputOk()) {
					synchronized (GUISungkaPlayerView.this) {
						GUISungkaPlayerView.this.retDialog = dialog;
						GUISungkaPlayerView.this.notifyAll();
					} // synchronized (GUISungkaPlayerView.this)
				} // if 
			} // actionPerformed
		}; // AbstractAction

		/** The Action for exiting the program */
		Action exitAction = new AbstractAction(resources.getString("GUI_sungkamenu_exit")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_X));
			}
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				dispose();
			}
		}; // AbstractAction

		/** The Action for disconnecting from the server */
		Action disconnectAction = new AbstractAction(resources.getString("GUI_sungkamenu_disconnect")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_D));
			}
			public void actionPerformed(ActionEvent e) {
				if (myModel.myBoard != null) {
					try {
						myModel.myBoard.exitGame();
					} catch (Exception f) {
					}
				}
				processDisconnect();
			}
		};

		/** The Action for starting the game server */
		Action startServerAction = new AbstractAction(resources.getString("GUI_servermenu_start")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
			}
			public void actionPerformed(ActionEvent e) {
				SungkaRegisterDialog dialog = new SungkaRegisterDialog(SungkaPlayerGUI.this);
				while (true) {
					dialog.setVisible(true);
					// the starting of the server is one of the conditions of
					// the if. If successful start, close the dialog, otherwise
					// keep it around and redisplay
					if (!dialog.inputOk() ||
						startServer(dialog.server, dialog.port, dialog.numPlayers, dialog.numCups, dialog.numStones)) {
						dialog.dispose();
						break;
					}
					JOptionPane.showMessageDialog(dialog, resources.getString("GUI_server_errormsg"),
							resources.getString("GUI_server_errortitle"), JOptionPane.ERROR_MESSAGE);
				} // if 
			}
		};

		/** The Action for stopping the game server.*/
		Action stopServerAction = new AbstractAction(resources.getString("GUI_servermenu_stop")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
			}
			public void actionPerformed(ActionEvent e) {
				stopServer(false);
			}
		};

		/** The Action for displaying the server log dialog */
		Action showServerLogAction = new AbstractAction(resources.getString("GUI_servermenu_log")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_L));
			}
			public void actionPerformed(ActionEvent e) {
				logDialog.setVisible(true);
			}
		};
		
		/** The Action for sending a chat message. */
		Action chatAction;

		/** Constructs as new GUI. */
		SungkaPlayerGUI() {
			super("SungkaPlayerGUI");

			// Build pane for game board
			JPanel tempBoardPane = new JPanel();
			// Set preferred size for initial layout. Removed when Layout manager is added.
			tempBoardPane.setPreferredSize(new Dimension(400, 150));

			// Build pane for game log
			gameLog = new JTextPane();
			gameLog.setEditable(false);
			initStyles(gameLog);
			JScrollPane scrollPaneForLog = new JScrollPane(gameLog);
			scrollPaneForLog.setPreferredSize(new Dimension(400, 200));
			gameLogView = scrollPaneForLog.getViewport();

			// Initialize pane that will contain the event log, the chat panel,
			// and rhe ready button
			JPanel logChatPanel = new JPanel();
			GridBagLayout logChatLayout = new GridBagLayout();
			logChatPanel.setLayout(logChatLayout);
			GridBagConstraints logChatConstraints = new GridBagConstraints();
			logChatConstraints.weightx = 1.0;
			logChatConstraints.weighty = 1.0;
			logChatConstraints.gridwidth = GridBagConstraints.REMAINDER;
			logChatConstraints.fill = GridBagConstraints.BOTH;
			logChatLayout.setConstraints(scrollPaneForLog, logChatConstraints);
			logChatPanel.add(scrollPaneForLog);

			// Initialize chat field
			final JTextField chatField = new JTextField();
			logChatConstraints.fill = GridBagConstraints.BOTH;
			logChatConstraints.weighty = 0.0;
			logChatConstraints.gridwidth = 1;
			logChatConstraints.gridy = GridBagConstraints.RELATIVE;
			logChatLayout.setConstraints(chatField, logChatConstraints);
			logChatPanel.add(chatField);

			// the Action for when the user clicks on Chat, or presses enter.
			chatAction = new AbstractAction(resources.getString("GUI_chat")) {
				{
					putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
				}
				public void actionPerformed(ActionEvent e) {
					final String msg = chatField.getText();
					myModel.queueMessage(new Runnable() {
						public void run() {
							myModel.myBoard.sendChatMessage(msg, -1);
						}
					});
					chatField.setText("");
				}
			};

			chatField.setAction(chatAction);
			chatAction.setEnabled(false);

			// The Chat button
			JButton chatButton = new JButton(chatAction);
			logChatConstraints.fill = GridBagConstraints.NONE;
			logChatConstraints.weightx = 0.0;
			logChatConstraints.gridx = GridBagConstraints.RELATIVE;
			logChatLayout.setConstraints(chatButton, logChatConstraints);
			logChatPanel.add(chatButton);

			// The ready to play button
			readyButton = new JButton();
			Action readyAction = new AbstractAction(resources.getString("GUI_ready")) {
				{
					putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_R));
				}
				public void actionPerformed(ActionEvent e) {
					myModel.queueMessage(new Runnable() {
						public void run() {
							myModel.myBoard.readyToPlay();
						}
					});
					log(resources.getString("waiting"), LOG_MOVE);
					readyButton.setVisible(false);
				}
			};
			readyButton.setAction(readyAction);
			readyButton.setVisible(false);
			logChatConstraints.fill = GridBagConstraints.NONE;
			logChatConstraints.gridx = 0;
			logChatConstraints.gridwidth = GridBagConstraints.REMAINDER;
			logChatConstraints.weightx = 0.0;
			logChatLayout.setConstraints(readyButton, logChatConstraints);
			logChatPanel.add(readyButton);
			
			// Build pane for player list
			playerModel = new SungkaListModel();
			playerList = new JList(playerModel);
			playerList.setCellRenderer(new SungkaListCellRenderer());
			playerList.setFont(new Font("Dialog", Font.PLAIN, 12));
			playerList.setPrototypeCellValue("A typical player name");
			JScrollPane scrollPaneForPlayerList = new JScrollPane(playerList);

			// Combine game log and game board into a split pane
			sungkaPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, tempBoardPane, logChatPanel);
			sungkaPane.setOneTouchExpandable(true);
			sungkaPane.setContinuousLayout(true);

			// Combine above split pane and player list into yet another split pane
			JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, scrollPaneForPlayerList, sungkaPane);
			splitPane.setOneTouchExpandable(true);
			splitPane.setContinuousLayout(true);

			splitPane.getActionMap().put("connect", connectAction);

			// Initialize status bars
			JPanel statusBar = new JPanel();
			GridBagLayout g = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			c.gridy = 0;
			c.weightx = 1.0;
			statusBar.setLayout(g);
			statusLabel = new JLabel(resources.getString("GUI_status_default"), JLabel.LEFT);
			Font f = statusLabel.getFont();
			statusLabel.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
			statusLabel.setBorder(BorderFactory.createLoweredBevelBorder());
			c.fill = GridBagConstraints.HORIZONTAL;
			g.setConstraints(statusLabel, c);
			statusBar.add(statusLabel);
			
			// Server status bar
			serverLabel = new JLabel(resources.getString("server_state0"), JLabel.RIGHT);
			c.weightx = 0.0;
			f = serverLabel.getFont();
			serverLabel.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
			serverLabel.setBorder(BorderFactory.createLoweredBevelBorder());
			serverLabel.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1) {
						if (server == null) {
							startServerAction.actionPerformed(new ActionEvent(this, 1, "label"));
						} else {
							showServerLogAction.actionPerformed(new ActionEvent(this, 1, "label"));
						}
					}
				}
			});
			g.setConstraints(serverLabel, c);
			statusBar.add(serverLabel);

			JPanel tempPanel = new JPanel();
			tempPanel.setLayout(new BorderLayout());
			tempPanel.add(splitPane, BorderLayout.CENTER);
			tempPanel.add(statusBar, BorderLayout.SOUTH);
			setContentPane(tempPanel);

			// Initialize menu
			menuBar = new JMenuBar();
			setJMenuBar(menuBar);

			// Initialize Sungka Menu
			JMenu menu = new JMenu(resources.getString("GUI_sungkamenu"));
			menu.setMnemonic(KeyEvent.VK_S);
			menu.getAccessibleContext().setAccessibleDescription(resources.getString("GUI_sungkamenu_tooltip"));
			menuBar.add(menu);

			menu.add(new JMenuItem(connectAction));
			menu.add(new JMenuItem(disconnectAction));
			menu.add(new JMenuItem(exitAction));

			// Initialize Server Menu
			menu = new JMenu(resources.getString("GUI_servermenu"));
			menu.setMnemonic(KeyEvent.VK_V);
			menu.getAccessibleContext().setAccessibleDescription(resources.getString("GUI_servermenu_tooltip"));
			menuBar.add(menu);

			menu.add(new JMenuItem(startServerAction));
			menu.add(new JMenuItem(stopServerAction));
			menu.add(new JMenuItem(showServerLogAction));
		} // SungkaPlayerGUI

		/**
		 * Initializes the text styles for the different kind of messages
		 * logged.
		 * @param textPane The pane displaying the game log messages.
		 */
		protected void initStyles(JTextPane textPane) {
			Style def = StyleContext.getDefaultStyleContext().
				getStyle(StyleContext.DEFAULT_STYLE);
			Style regular = textPane.addStyle("regular", def);

			Style s = textPane.addStyle(LOG_ERROR, regular);
			StyleConstants.setForeground(s, Color.WHITE);
			StyleConstants.setBackground(s, Color.RED);
			StyleConstants.setBold(s, true);

			s = textPane.addStyle(LOG_MOVE, regular);

			s = textPane.addStyle(LOG_PLAYER, regular);
			StyleConstants.setForeground(s, Color.BLUE);

			s = textPane.addStyle(LOG_CHAT, regular);
			StyleConstants.setForeground(s, Color.GREEN);
			StyleConstants.setBold(s, true);

			s = textPane.addStyle(LOG_DEBUG, regular);
			StyleConstants.setForeground(s, Color.LIGHT_GRAY);

			s = textPane.addStyle(LOG_GAMEERROR, regular);
			StyleConstants.setForeground(s, Color.RED);
			StyleConstants.setBold(s, true);
		} // initStyles

		/**
		 * Creates a new game board pane and initializes it from the model.
		 * @param observer True if player is an observer, false if participant.
		 */
		protected void initBoard(boolean observer) {
			boardPane = new SungkaBoardPane(observer);
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					sungkaPane.setLeftComponent(boardPane);
				}
			});
			if (!observer) {
				readyButton.setVisible(true);
			}
		} // initBoard

		/**
		 * Notifies the GUI when a player is added.
		 * @param s A SungkaPlayerInfo with info about the added player.
		 */
		protected void playerAdded(SungkaPlayerInfo s) {
			playerModel.playerAdded(s);
			boardPane.playerAdded(s);
		}

		/**
		 * Notifies the GUI when a player is removed.
		 * @param s A SungkaPlayerInfo with info about the departed player.
		 */
		protected void playerRemoved(SungkaPlayerInfo s) {
			playerModel.playerRemoved(s);
			boardPane.playerRemoved(s);
		}

		/**
		 * Notifies the GUI when a player's turn is starting.
		 * @param player The player number of the player whose turn is starting.
		 */
		protected void startTurn(int player) {
			SungkaPlayerInfo s = myModel.getPlayerInfo(player);
			// update list
			playerModel.startTurn(s);
			// update board
			boardPane.startTurn(s);
		}

		/**
		 * Notifies the GUI when a player's turn is ending.
		 * @param player The player number of the player whose turn is ending.
		 */
		protected void endTurn(int player) {
			SungkaPlayerInfo s = myModel.getPlayerInfo(player);
			// update list
			playerModel.endTurn(s);
			// update board
			boardPane.endTurn(s);
		}

		/**
		 * Notifies the GUI when the contents a cup has been updated.
		 * @param side The side whose cup value has changed.
		 * @param cup The cup number of the cup whose value changed
		 */
		protected void updateCup(final int side, final int cup) {
			boardPane.updateCup(side, cup);
		}

		/**
		 * Resets the GUI to startup state. This will allow the GUI to be ready
		 * to connect to another server.
		 */
		protected void gameReset() {
			// Delte old board. Will be replaced later
			if (boardPane != null) {
				boardPane = null;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						JPanel tempPane = new JPanel();
						tempPane.setPreferredSize(new Dimension(400, 150));
						sungkaPane.setLeftComponent(tempPane);
					}
				});
			}
			// Reset player list
			playerModel.gameReset();
		}
	} // class SungkaPlayerGUI

	/**
	 * SungkaListCellRenderer implements a list cell renderer for
	 * SungkaPlayerInfo objects. The renderer will also highlight the current
	 * player in a JList.
	 */
	class SungkaListCellRenderer extends DefaultListCellRenderer {
		/**
		 * Return a component that has been configured to display the specified
		 * SungkaPlayerInfo. That component's paint method is then called to
		 * "render" the cell. If the player being painted is the current one,
		 * this renderer will cause the component to be highlighted.
		 * @param list The JList we're painting.
		 * @param value The value returned by list.getModel().getElementAt(index).
		 * @param index The cells index.
		 * @param isSelected True if the specified cell was selected.
		 * @param cellHasFocus True if the specified cell has the focus.
		 */
		public Component getListCellRendererComponent(JList list, Object value,
				 int index, boolean isSelected, boolean cellHasFocus) {
			if (value instanceof SungkaPlayerInfo) {
				Component c = super.getListCellRendererComponent(list,
						((SungkaPlayerInfo)value).getName(),
						index, isSelected, cellHasFocus);
				// Turn it bolded red if it's the current player
				if (((SungkaPlayerInfo)value).getPlayerNumber() == myModel.getCurrentPlayer()) {
					c.setForeground(Color.RED);
					Font f = c.getFont();
					c.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
				}
				return c;
			} else {
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus); 
			}
		}
	} // class SungkaListCellRenderer

	/**
	 * SungkaBoardPane is a panel that displays the cups of the Sungka board.
	 * It holds a collection of SungkaCup objects which will do the actual
	 * display, and lays them out on the display.
	 */
	class SungkaBoardPane extends JPanel {
		/** The array of cups */
		SungkaCup[][] myCups;
		/** The names of the participants. */
		JLabel[] myNames;

		/**
		 * Constructs a new SungkaBoardPane.
		 * @param observer True if this player is an observer
		 *                 (for setting cups active)
		 */
		SungkaBoardPane(boolean observer) {
			int numCups = myModel.getNumCups();
			int numPlayers = myModel.getNumParticipants();

			setPreferredSize(new Dimension(400, 150));
			myCups = new SungkaCup[numPlayers][numCups + 1];
			myNames = new JLabel[numPlayers];

			GridBagLayout gridBag = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			setLayout(gridBag);
			c.fill = GridBagConstraints.BOTH;

			// The following is a possible special layout of home cups for a 2
			// player game
			/*
			JPanel homePanel = new SungkaCup("Home 1");
			c.gridx = 0;
			c.gridy = 0;
			c.gridheight = 2;
			gridBag.setConstraints(homePanel, c);
			myCups[1][7] = homePanel;
			add(homePanel);

			homePanel = new SungkaCup("Home 0");
			c.gridx = 8;
			gridBag.setConstraints(homePanel, c);
			myCups[0][7] = homePanel;
			add(homePanel);
			*/

			c.gridheight = 1; 
			for (int i = 0; i < numPlayers; i++) {
				c.weightx = 0.5;
				c.weighty = 0.5;
				c.gridy = (numPlayers - 1) - i;
				for (int j = 0; j < numCups + 1; j++) {
					SungkaCup cupPanel = new SungkaCup(j);
					// Lay out cups such that even player go left to right, and
					// odd players go right to left, alternating.
					c.gridx = (i % 2 == 0 ? j + 2: numCups - j + 1);
					gridBag.setConstraints(cupPanel, c);
					myCups[i][j] = cupPanel;
					add(cupPanel);
				} // for

				// Add a label on the end to identify which participant the row
				// belongs to
				myNames[i] = new JLabel(resources.getString("GUI_waiting"));
				myNames[i].setForeground(Color.GRAY);
				c.gridx = (i % 2 == 0 ? numCups + 3 : 0);
				c.weightx = (i % 2 == 0 ? -0.1 : 0.1);
				c.weighty = 0;
				gridBag.setConstraints(myNames[i], c);
				add(myNames[i]);
			} // for

			// If active participant, set our row (the 0th row) playable.
			if (!observer) {
				for (int i = 0; i < numCups; i++) {
					myCups[0][i].setPlayable();
				} // for
			} // if
			revalidate();
		} // SungkaBoardPane

		/** Processes updateBoard message, by updating contents of all cups. */
		protected void updateBoard() {
			for (int i = 0; i < myCups.length; i++) {
				for (int j = 0; j < myCups[i].length; j++) {
					updateCup(i, j);
				}
			}
		}

		/**
		 * Processes updateCup message, notifying the cup of the change.
		 * @param side The side which had the cup change value.
		 * @param cup The cup number of the cup whose value changed.
		 */
		protected void updateCup(int side, int cup) {
			myCups[side][cup].setStones(myModel.getBoard()[side][cup]);
		}

		/**
		 * Processes a playerAdded message. Will update display to associate
		 * the proper row with the player name.
		 * @param s A SungkaPlayerInfo with info about the new player.
		 */
		protected void playerAdded(SungkaPlayerInfo s) {
			if (!s.isObserver()) {
				final int order = s.getOrder();
				final String name = s.getName();
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						myNames[order].setText(name);
						myNames[order].setForeground(Color.BLACK);
					}
				});
			}
		}

		/**
		 * Processes a playerAdded message. Will update display to dissociate
		 * the proper row with the player name.
		 * @param s A SungkaPlayerInfo with info about the departing player.
		 */
		protected void playerRemoved(SungkaPlayerInfo s) {
			if (!s.isObserver()) {
				final int order = s.getOrder();
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						myNames[order].setText(resources.getString("GUI_waiting"));
						myNames[order].setForeground(Color.GRAY);
					}
				});
			}
		}

		/**
		 * Processes a startTurn message. Will update display to set the proper
		 * row active.
		 * @param s A SungkaPlayerInfo with info about the player whose turn is
		 *          starting.
		 */
		protected void startTurn(SungkaPlayerInfo s) {
			int order = s.getOrder();
			for (int i = 0; i < myCups[order].length; i++) {
				myCups[order][i].setActive(true);
			}
		}


		/**
		 * Processes a endTurn message. Will update display to set the proper
		 * row inactive.
		 * @param s A SungkaPlayerInfo with info about the player whose turn is
		 *          ending.
		 */
		protected void endTurn(SungkaPlayerInfo s) {
			int order = s.getOrder();
			for (int i = 0; i < myCups[order].length; i++) {
				myCups[order][i].setActive(false);
			}
		}
	} // class SungkaBoardPane

	/**
	 * SungkaCup is the actual cup on the GUI board. This class handles
	 * display of the number of stones, the user clicks to tell their move,
	 * and highlighting the cup when it is the user's turn.
	 */
	class SungkaCup extends JPanel {
		/** The number of stones in this cup. */
		int numStones = 0;
		/** The cup number of this cup. */
		int cupNum;
		/** True if this cup will be highlighted and accept user's clicks. */
		boolean active = false;
		/** The label showing the number of stones. */
		JLabel myLabel;

		/**
		 * Constructs a new SungkaCup.
		 * @param cupNum The cup number of this cup, to identify the cup to the
		 *               game board when the user clicks on the cup.
		 */
		public SungkaCup(int cupNum) {
			super();
			this.cupNum = cupNum;
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			// Add the label. To be replaced by actual stones one day.
			myLabel = new JLabel(cupNum + "");
			// Center it
			myLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
			add(Box.createVerticalGlue());
			add(myLabel);
			add(Box.createVerticalGlue());
		} // SungkaCup

		/** Paints this cup. Draws the cup, and if active, highlights it. */
		public void paintComponent(Graphics g) {
			// paint background
			super.paintComponent(g);

			Dimension myDim = getSize();
			Graphics2D g2 = (Graphics2D)g;
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setStroke(new BasicStroke(2.0f));

			if (active) {
				// highlight it
				g2.setPaint(Color.WHITE);
				g2.fill(new Ellipse2D.Double(0, 0, myDim.width, myDim.height));
			}

			// draw the cup outline
			g2.setPaint(Color.CYAN);
			g2.draw(new Ellipse2D.Double(0, 0, myDim.width, myDim.height));
		} // paintComponent

		/** Sets the number of stones in the cup and updates the GUI. */
		public void setStones(final int num) {
			numStones = num;
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					myLabel.setText(num + "");
				}
			});
		} // setStones

		/**
		 * Makes this cup is playable. If the cup is playable, it will tell the
		 * model to make a move when it is clicked, giving its cup number.
		 */
		public void setPlayable() {
			setFocusable(true);
			addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (active && e.getButton() == MouseEvent.BUTTON1) {
						myModel.queueMessage(new Runnable() {
							public void run() {
								if (!myModel.myBoard.makeMove(cupNum)) {
									log(resources.getString("badmove"), LOG_GAMEERROR);
								} // if
							}
						});
					} // if
				} // mouseClicked
			}); // MouseAdapter
		} // setPlayable

		/**
		 * Sets whether this cup is active or not. If it is active, the cup
		 * will be highlighted, and if playable accepts user clicks.
		 * @param b True to set this cup active, false for inactive.
		 */
		public void setActive(boolean b) {
			if (active != b) {
				active = b;
				repaint();
			}
		}
	} // class SungkaCup
} // class GUISungkaPlayerView
