/*
 * Sungka v0.10.
 * SungkaBoardServant.java - Player interface to game board
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import org.omg.PortableServer.POA;

/**
 * Implements the SungkaBoard interface, allowing players to interact with the
 * game. A game player receives a reference to a SungkaBoard as its interface
 * to the game, and calls its methods to communicate any moves, requests, or
 * other messages it makes.
 *
 * Internally, a SungkaBoardServant stores the player number and passes it on
 * to the board methods, so that the board knows which player called it.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
class SungkaBoardServant extends SungkaBoardPOA {
	/** The game board associated with this player. */
	protected SungkaRegisterServant regServant;

	/** The reference to the remote player. */
	protected SungkaPlayer playerRef;

	/** This player's player number. */
	protected int player;

	/** The name of this player. */
	protected String name;

	/** A cached reference to ourself. */
	protected SungkaBoard href;

	/** Stores whether player has already notified the board that it's ready. */
	protected boolean isReady = false;

	/**
	 * Constructs a new SungkaBoardServant.
	 * @param regServant The game board to associate with.
	 * @param playerRef A reference to the remote player.
	 * @param player The player number to assign to this player.
	 * @param name The name of this player.
	 * @param poa A CORBA Portable Object Adapter to get a self-reference from.
	 */
	SungkaBoardServant(SungkaRegisterServant regServant, SungkaPlayer playerRef, int player, String name, POA poa) {
		this.regServant = regServant;
		this.playerRef = playerRef;
		this.player = player;
		this.name = name;
		try {
			href = SungkaBoardHelper.narrow(poa.servant_to_reference(this));
		} catch (org.omg.PortableServer.POAPackage.ServantNotActive e) {
			e.printStackTrace(regServant.log);
			System.exit(1);
		} catch (org.omg.PortableServer.POAPackage.WrongPolicy e) {
			e.printStackTrace(regServant.log);
			System.exit(1);
		}
	}

	/** Requests the current game parameters from the game. */
	public void requestParams() {
		regServant.requestParams(this);
	}

	/** Requests the current board contents from the game. */
	public void requestBoard() {
		regServant.requestBoard(this);
	}

	/**
	 * Makes a game move.
	 * @param cup The cup we are making the move from.
	 * @return True if the move was valid and accepted, false otherwise
	 */
	public boolean makeMove(int cup) {
		return regServant.makeMove(player, cup);
	}

	/** Notifies the game board that the player is ready to begin the game. */
	public synchronized void readyToPlay() {
		// synchronize with reset FIXME need better variable
		synchronized (regServant.players) {
			if (!isReady) {
				regServant.readyToPlay(this);
				isReady = true;
			}
		}
	}

	/**
	 * Sends a chat message to other players in the game.
	 * @param msg The actual message.
	 * @param destPlayer The destination player number to send to. Negative to
	 *                   broadcast to all. NOT IMPLEMENTED (ignored for now)
	 */
	public void sendChatMessage(String msg, int destPlayer) {
		// FIXME add support for destination player!
		regServant.sendChatMessage(player, msg);
	}

	/**
	 * Receives ping message from the Player.
	 * Calling this function is used simply to check the CORBA link, and does not require any
	 * processing other than the CORBA response. Body can be left empty.
	 */
	public void ping() {
	}

	/** Asks that this player exit the game and disconnect from the server. */
	public void exitGame() {
		regServant.exitGame(this);
	}
}
