/*
 * Sungka v0.10.
 * SungkaConnectDialog.java - Connection Dialog box
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Implements the connection dialog for the GUI View. A user uses this dialog
 * to specify which server to connect to, user name, and whether to be an
 * observer or not.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class SungkaConnectDialog extends JDialog implements SungkaConstants {
	/** True if OK button was pressed and data fields are valid. */
	private boolean inputOk = false;
	
	/**
	 * The server name to connect to. The game board reads this if it
	 * determines that the input is valid.
	 */
	String server = null;

	/**
	 * The port to connect to. The game board reads this if it determines
	 * that the input is valid.
	 */
	int port = 0;

	/**
	 * The name of this player. The game board reads this if it determines
	 * that the input is valid.
	 */
	String playerName = null;

	/**
	 * True if user wants to be an observer to this game. The game board reads
	 * this if it determines that the input is valid.
	 */
	boolean observer = false;

	/** The text field to input the server to connect to. */
	private final JTextField serverField = new JTextField(DEFAULT_HOST, 30);

	/** The text field to input the port to connect to. */
	private final JTextField portField = new JTextField(DEFAULT_PORT + "", 4);

	/** The text field to input the user name. */
	private final JTextField nameField = new JTextField("Observer", 16);

	/** The check box to be an observer or active participant. */
	private final JCheckBox observerCheck =
		new JCheckBox(resources.getString("GUI_connect_observer"));

	/** The label for the server field. */
	private final JLabel serverLabel =
		new JLabel(resources.getString("GUI_connect_server")); 
	/** The label for the port field. */
	private final JLabel portLabel =
		new JLabel(resources.getString("GUI_connect_port")); 
	/** The label for the user name field. */
	private final JLabel nameLabel =
		new JLabel(resources.getString("GUI_connect_user"));

	/**
	 * Constructs a new connection dialog.
	 * @param parentWindow The parent window of this dialog.
	 */
	SungkaConnectDialog(Frame parentWindow) {
		super(parentWindow, resources.getString("GUI_connect_title"), true);
		inputOk = false;
		JPanel content = new JPanel();
		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		content.setLayout(gridBag);
		c.fill = GridBagConstraints.NONE;

		// The action for the OK button
		Action okAction = new AbstractAction(resources.getString("GUI_ok")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_K));
			}
			public void actionPerformed(ActionEvent e) {

				// extract input
				String portString = portField.getText();
				server = serverField.getText();
				playerName = nameField.getText();
				observer = observerCheck.isSelected();

				// validate input
				String errorMessage = "";

				// validate server
				if (server.equals("")) {
					errorMessage += "\n- " +
						resources.getString("GUI_connect_badserver");
					serverLabel.setForeground(errorColor);
					serverField.requestFocus();
				} else {
					serverLabel.setForeground(defaultColor);
				}

				// validate port
				try {
					port = Integer.parseInt(portString);
					portLabel.setForeground(defaultColor);
				} catch (NumberFormatException f) {
					errorMessage += "\n- " +
						resources.getString("GUI_connect_badport");
					portLabel.setForeground(errorColor);
					portLabel.requestFocus();
				}

				// validate user name
				if (playerName.equals("")) {
					errorMessage += "\n- " +
						resources.getString("GUI_connect_emptyuser");
					nameLabel.setForeground(errorColor);
					nameField.requestFocus();
				} else {
					nameLabel.setForeground(defaultColor);
				} // if

				if (errorMessage.equals("")) {
					inputOk = true;
					setVisible(false);
				} else {
					// reset to defaults, just in case
					port = 0;
					server = null;
					playerName = null;
					observer = false;

					// Show error dialog
					JOptionPane.showMessageDialog(SungkaConnectDialog.this,
						resources.getString("GUI_connect_pleasefix") + "\n" +
						errorMessage,
						resources.getString("GUI_connect_errortitle"),
						JOptionPane.WARNING_MESSAGE);
				} // if
			} // actionPerformed
		}; // Abstractaction

		// The Action for the Cancel button.
		Action cancelAction = new AbstractAction(resources.
				getString("GUI_cancel")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
			}
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		}; // AbstractAction

		// Add components

		// Server label (Alt-S)
		c.gridy = 0;
		serverLabel.setLabelFor(serverField);
		serverLabel.setDisplayedMnemonic(KeyEvent.VK_S);
		c.gridx = 0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.EAST;
		gridBag.setConstraints(serverLabel, c);
		serverLabel.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
		content.add(serverLabel);
		// Server textfield
		c.gridx = 1;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridBag.setConstraints(serverField, c);
		content.add(serverField);

		// Port label (Alt-P)
		portLabel.setLabelFor(portField);
		portLabel.setDisplayedMnemonic(KeyEvent.VK_P);
		c.gridx = 3;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.EAST;
		gridBag.setConstraints(portLabel, c);
		portLabel.setBorder(BorderFactory.createEmptyBorder(0,10,0,5));
		content.add(portLabel);
		c.gridx = 4;
		// Port textfield
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.WEST;
		gridBag.setConstraints(portField, c);
		content.add(portField);

		c.gridy = 1;
		// User label (Alt-U)
		nameLabel.setLabelFor(nameField);
		nameLabel.setDisplayedMnemonic(KeyEvent.VK_U);
		c.gridx = 0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.EAST;
		gridBag.setConstraints(nameLabel, c);
		nameLabel.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
		content.add(nameLabel);
		// User textfield
		c.gridx = 1;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.WEST;
		gridBag.setConstraints(nameField, c);
		content.add(nameField);

		// Observer checkbox (Alt-O)
		observerCheck.setMnemonic(KeyEvent.VK_O);
		observerCheck.setHorizontalTextPosition(JCheckBox.LEADING);
		c.gridx = 2;
		c.gridwidth = 3;
		c.anchor = GridBagConstraints.EAST;
		gridBag.setConstraints(observerCheck, c);
		content.add(observerCheck);

		c.gridy = 2;
		// OK Button (Alt-K or Enter)
		JButton okButton = new JButton(okAction);
		okButton.setHorizontalTextPosition(AbstractButton.CENTER);
		c.gridx = 1;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.CENTER;
		gridBag.setConstraints(okButton, c);
		content.add(okButton);

		// Cancel Button (Alt-C or Escape)
		JButton cancelButton = new JButton(cancelAction);
		cancelButton.setHorizontalTextPosition(AbstractButton.CENTER);
		c.gridx = 2;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.CENTER;
		gridBag.setConstraints(cancelButton, c);
		content.add(cancelButton);

		// Map escape and enter to cancel and okay's actions
		// (NOT to the actual buttons)
		content.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,
					0), "cancel");
		content.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
				"ok");
		content.getActionMap().put("ok", okAction);
		content.getActionMap().put("cancel", cancelAction);

		content.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		setContentPane(content);

		setResizable(false);
		pack();
		setLocationRelativeTo(parentWindow);

		addWindowListener(new WindowAdapter() {
			private boolean gotFocus = false;

			public void windowClosing(WindowEvent we) {
				inputOk = false;
			}

			public void windowGainedFocus(WindowEvent we) {
				// Once window gets focus, set initial focus
				if (!gotFocus) {
					// nameField.requestFocus();
					gotFocus = true;
				}
			} // windowGainedFocus
		}); // WindowAdapter
	} // SungkaConnectDialog

	/**
	 * Tells whether the input was valid. (e.g. the user hit the OK button.)
	 * @return True if the input was valid
	 */
	public boolean inputOk() {
		return inputOk;
	}
} // class SungkaConnectDialog
