/*
 * Sungka v0.10.
 * SungkaListModel.java - Player List Model
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import java.util.*;
import javax.swing.*;

/**
 * SungkaListModel implements the List Model for a Swing JList of players.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
class SungkaListModel extends AbstractListModel {
	/** Our internal list of players, sorted by player number. */
	private ArrayList playerArray = new ArrayList();

	/**
	 * Returns the size of the list.
	 * @return The size of the list.
	 */
	public int getSize() {
		return playerArray.size();
	}

	/**
	 * Returns the player at the specified row.
	 * @param row The requested row
	 * @return The player at row
	 */
	public Object getElementAt(int row) {
		return (SungkaPlayerInfo)playerArray.get(row);
	}

	/**
	 * Notifies this list model of an added player. This function will sort
	 * the player into its proper position, then notify the JList of the
	 * change.
	 * @param s A SungkaPlayerInfo with info about the new player
	 */
	void playerAdded(SungkaPlayerInfo s) {
		final int index = -Collections.binarySearch(playerArray, s) - 1;
		playerArray.add(index, s);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireIntervalAdded(this, index, index);
			}
		});
	}

	/**
	 * Notifies this list model of a departing player. This function will find
	 * the player in the list, remote it, then notify the JList of the change.
	 * @param s A SungkaPlayerInfo with info about the removed player
	 */
	void playerRemoved(SungkaPlayerInfo s) {
		final int index = Collections.binarySearch(playerArray, s);
		playerArray.remove(index);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireIntervalRemoved(this, index, index);
			}
		});
	}

	/**
	 * Notifies the list model of a player starting its turn. The method will
	 * find the player in the list, then notify the JList of the change. It is
	 * up to a cell renderer to actually change the appearance of the row.
	 * @param s A SungkaPlayerInfo with info about the player starting its turn
	 */
	void startTurn(SungkaPlayerInfo s) {
		final int index = Collections.binarySearch(playerArray, s);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireContentsChanged(this, index, index);
			}
		});
	}

	/**
	 * Notifies the list model of a player ending its turn. The method will
	 * find the player in the list, then notify the JList of the change. It is
	 * up to a cell renderer to actually change the appearance of the row.
	 * @param s A SungkaPlayerInfo with info about the player ending its turn
	 */
	void endTurn(SungkaPlayerInfo s) {
		final int index = Collections.binarySearch(playerArray, s);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireContentsChanged(this, index, index);
			}
		});
	}

	/**
	 * Notifies the list model of the game state being reset. This function
	 * will remove all players to be ready to accept new players.
	 */
	void gameReset() {
		if (playerArray.size() > 0) {
			SwingUtilities.invokeLater(new Runnable() {
				final int size = getSize();
				public void run() {
					fireIntervalRemoved(this, 0, size);
				}
			});
			playerArray.clear();
		}
	}
} // class SungkaListModel
