/*
 * Sungka v0.10.
 * Player class
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import org.omg.CORBA.ORB;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.INTERNAL;
import org.omg.CORBA.IntHolder;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POA;

import java.util.*;
import java.text.MessageFormat;

/**
 * SungkaPlayerServant implements the player side of the game. This class acts
 * as a controller that communicates with the game board, and receives messages
 * from it. For a user to actually play, a SungkaPlayerView must be registered
 * with us.
 * 
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class SungkaPlayerServant extends SungkaPlayerPOA implements SungkaConstants {

	/** Our copy of the actual playing board. */
	protected int[][] board;

	/** A map from player number to player info data structure */
	protected Map playerMap = new HashMap();

	/** The current player */
	protected int currentPlayer = -1;

	/** The number of participants in the game. */
	protected int numParticipants = -1;

	/** The number of cups per participant. */
	protected int numCups = -1;

	/**
	 * A semaphore to determine when to disconnect from the game. When set to
	 * true, the main event loop will initiate a disconnect.
	 */
	protected volatile boolean disconnect = false;

	/** This player's number. */
	protected int myNumber = -1;

	/**
	 * A queue of messages to be processed. Messages from the SungkaBoard get
	 * queued here.
	 */
	protected List messages = Collections.synchronizedList(new LinkedList());

	/** ORB Reference to the SungkaBoard we are playing on */
	protected SungkaBoard myBoard;

	/** ORB reference to myself. */
	protected SungkaPlayer myRef = null;

	/** The view of the game. */
	private SungkaPlayerView myView = null;

	/** Constructs a new SungkaPlayerServant. */
	public SungkaPlayerServant() {
	}

	/**
	 * Sets the view of this player.
	 * @param view The SungkaPlayerView to associate with this player.
	 */
	public void setView(SungkaPlayerView view) {
		myView = view;
	}

	/** Starts the event processing loop of this Sungka player. */
	public void mainLoop() {
		while (!disconnect) {
			// Wait for messages to be received from the SungkaBoard
			if (messages.size() > 0) {
				Runnable r = (Runnable)messages.remove(0);
				try {
					r.run();
				} catch (COMM_FAILURE e) {
					disconnect();
				}
			} else synchronized (this) {
				// FIXME There is a race condition here where a message is
				// queued after the disconnect, but before the else. Not
				// critical since we'll pick it up in 10 seconds on the ping,
				// run but stil worth fixing.
				try {
					// Sleep if message queue is empty
					wait(10000);
				} catch (InterruptedException e) {
					e.printStackTrace();
					System.exit(1);
				}
				queueMessage(new Runnable() {
					public void run() {
						myBoard.ping();
					}
				});
			}
		}
	}

	/**
	 * Queues a message to be processed, and notifies the event loop.
	 * @param r A Runnable containining a message to be processed.
	 */
	protected void queueMessage(Runnable r) {
		messages.add(r);
		synchronized (this) {
			notifyAll();
		}
	}

	/**
	 * Returns the current board contents.
	 * @return The current board contents.
	 */
	public int[][] getBoard() {
		return board;
	}

	/**
	 * Returns the number of cups per player in the current game.
	 * @return The number of cups per player.
	 */
	public int getNumCups() {
		return numCups;
	}

	/**
	 * Returns the number of participants in the current game.
	 * @return The number of participants.
	 */
	public int getNumParticipants() {
		return numParticipants;
	}

	/**
	 * Returns whether or not this player is an observer.
	 * @return true if this player is an observer
	 */
	public boolean isObserver() {
		return myNumber < 0;
	}

	/**
	 * Returns a SungkaPlayerInfo object containing information about the
	 * player with the given player number.
	 * @param player The player number which we want to get info about.
	 * @return A SungkaPlayerInfo object with info about the player.
	 */
	public SungkaPlayerInfo getPlayerInfo(int player) {
		return (SungkaPlayerInfo)playerMap.get(new Integer(player));
	}

	/**
	 * Resets this game to as close to game startup state as possible.
	 * Notifies the view of the game reset so that it can do any necessary
	 * processing.
	 */
	protected void resetGame() {
		board = null;
		disconnect = false;
		playerMap.clear();
		currentPlayer = -1;
		numParticipants = -1;
		numCups = -1;
		myNumber = -1;
		messages.clear();
		myBoard = null; // Assume disconnect releases old one
		myRef = null; // Assume disconnect releases old one
		myView.gameReset();
	}

	/**
	 * Returns the player number of this player.
	 * @return the player number of this player.
	 */
	protected int getMyNumber() {
		return myNumber;
	}

	/**
	 * Returns the player number of the current player making a move.
	 * @return The player number of the current player making a move, or -1 if
	 *         currently nobody's turn.
	 */
	protected int getCurrentPlayer() {
		return currentPlayer;
	}

	/**
	 * Checks whether it is this players turn.
	 * @return True if it is my turn, false otherwise.
	 */
	protected boolean myTurn() {
		return currentPlayer == myNumber;
	}

	/**
	 * Connects to a game board at the specified server and port.
	 * This method will first try to resolve using a CORBA Initial Naming
	 * Service listening on that server and port. If that fails, it will
	 * then try to resolve using the CORBAservice Interoperable Naming Service
	 * listening on the same port.
	 * @param server The server to connect to
	 * @param port The port to connect to
	 * @return A reference to a game board registration object, or null if we
	 *         were unsuccessful.
	 */
	protected SungkaRegister connectToSungka(String server, int port) {
		try {
			// resolve the Object Reference in Naming
			String name = "Sungka";
			SungkaRegister sungkaServant = null;
			try {
				try {
					sungkaServant = SungkaRegisterHelper.narrow(_orb().
							string_to_object("corbaloc::" + server + ":" + port + "/" + name));
				} catch (COMM_FAILURE e) {
					e.printStackTrace();
				} catch (INTERNAL e) {
					if ((e.minor & 0x00000FFF) != 202) {
						throw e;
					}
				}

				if (sungkaServant == null) {
					try {
						sungkaServant = SungkaRegisterHelper.narrow(_orb().
								string_to_object("corbaname::" + server + ":" + port + "#" + name));
					} catch (COMM_FAILURE f) {
					}
				}
			} catch (BAD_PARAM e) {
				myView.log(resources.getString("CORBA_wrongtype"), LOG_ERROR);
				return null;
			}

			if (sungkaServant == null) {
				myView.log(resources.getString("CORBA_cantcontactgame"), LOG_ERROR);
			}

			return sungkaServant;
		} catch (Exception e) {
			myView.log(resources.getString("exception"), LOG_ERROR);
			e.printStackTrace();
			return null;
		} // try
	} // connectToSungka

	/**
	 * Joins a Sungka game.
	 *
	 * IMPORTANT NOTE: While joining a game, a SungkaPlayer, must be able to take messages,
	 * even though it's not fully initialized!
	 *
	 * @param s An ORB reference to the registration interface of a Sungka game.
	 * @param name My name.
	 * @param observer True if we want to be an observer.
	 */
	public boolean joinSungkaGame(SungkaRegister s, String name, boolean observer) {
		resetGame();

		SungkaPlayer ref = null;
		try {
			POA rootpoa = POAHelper.narrow(_orb().resolve_initial_references("RootPOA"));
			ref = SungkaPlayerHelper.narrow(rootpoa.servant_to_reference(this));
			rootpoa.the_POAManager().activate();
		} catch (Exception e) {
			myView.log(resources.getString("exception"), LOG_ERROR);
			e.printStackTrace();
			return false;
		}

		IntHolder i = new IntHolder(0);

		// Try to join a game and get a reference to the playing interface. If null, we were rejected.
		try {
			if ((myBoard = s.joinGame(ref, name, observer, i)) == null) {
				myView.log(resources.getString("cantjoin"), LOG_ERROR);
				return false;
			}
		} catch (COMM_FAILURE e) {
			myView.log(resources.getString("CORBA_cantcontactgame"), LOG_ERROR);
			return false;
		}

		myNumber = i.value;
		myRef = ref;
		//myView.joinedGame();
		return true;
	}

	/**
	 * Returns the players name for the given player number.
	 * @param player The player number who we want to get the name for.
	 * @return The corresponding players name.
	 */
	protected String getPlayerName(int player) {
		SungkaPlayerInfo s = (SungkaPlayerInfo)playerMap.get(new Integer(player));
		if (s != null) {
			return s.getName();
		} else {
			return null;
		}
	}

	/**
	 * Updates the game parameters of the current game.
	 * @param params An array with all the game parameters. The order of the
	 *               numbers is specified by constants in the SungkaBoard class
	 */
	public void updateParams(final int[] params) {
		queueMessage(new Runnable() {
			public void run() {
				numParticipants = params[SungkaBoard.PLAYER_COUNT];
				numCups = params[SungkaBoard.CUP_COUNT];
				currentPlayer = params[SungkaBoard.CURRENT_PLAYER];
				myView.processUpdateParams();
			}
		});
	}

	/**
	 * Updates the contents of the board of the current game.
	 * @param board A 2-D array containing the number of stones in each cup.
	 */
	public void updateBoard(final int[][] board) {
		queueMessage(new Runnable() {
			public void run() {
				SungkaPlayerServant.this.board = board;
				myView.processUpdateBoard();
			}
		});
	}

	/**
	 * Receives playerAdded message from the Board, and queues message for processing.
	 * @param name Name of new player
	 * @param player The player number of the new player
	 * @param order The order the new player will be playing, relative to other players.
	 */
	public void playerAdded(final String name, final int player, final int order) {
		queueMessage(new Runnable() {
			public void run() {
				SungkaPlayerInfo retVal = new SungkaPlayerInfo(name, player, order);
				playerMap.put(new Integer(player), retVal);
				myView.processPlayerAdded(retVal);
			}
		});
	}

	/**
	 * Receives playerRemoved message from the Board, and queues message for processing.
	 * @param player The player number of the exiting player
	 */
	public void playerRemoved(final int player) {
		queueMessage(new Runnable() {
			public void run() {
				myView.processPlayerRemoved((SungkaPlayerInfo)playerMap.remove(new Integer(player)));
			}
		});
	}

	/**
	 * Receives startTurn message from the Board, and queues message for processing.
	 * @param player The player number of the player starting his turn.
	 */
	public void startTurn(final int player) {
		queueMessage(new Runnable() {
			public void run() {
				currentPlayer = player;
				myView.processStartTurn(player);
			}
		});
	}

	/**
	 * Receives endTurn message from the Board, and queues message for processing.
	 * @param player The player number of the player ending his turn.
	 */
	public void endTurn(final int player) {
		queueMessage(new Runnable() {
			public void run() {
				currentPlayer = -1;
				myView.processEndTurn(player);
			}
		});
	}

	/**
	 * Receives playerMoved message from the Board, and queues message for
	 * processing.
	 * @param player The player number of the player making his move.
	 * @param cup The cup number the player is making his move from.
	 */
	public void playerMoved(final int player, final int cup) {
		queueMessage(new Runnable() {
			public void run() { myView.processPlayerMoved(player, cup); }
		});
	}

	/**
	 * Receives addToCup message from the Board, and queues message for
	 * processing.
	 * @param player The player number of the side receiving the stones.
	 * @param cup The cup number the stones are being added to.
	 * @param count The number of stones to add to this cup.
	 */
	public void addToCup(final int player, final int cup, final int count) {
		queueMessage(new Runnable() {
			public void run() {
				int side = ((SungkaPlayerInfo)playerMap.get(new Integer(player))).getOrder();
				board[side][cup] += count;
				myView.processUpdateCup(player, cup);
			}
		});
	}

	/**
	 * Receives setCup message from the Board, and queues message for
	 * processing. Also used to empty a cup.
	 * @param player The player number of the side receiving the stones.
	 * @param cup The cup number in which we are setting the number of stones.
	 * @param count The number of stones to set this cup to hold.
	 */
	public void setCup(final int player, final int cup, final int count) {
		queueMessage(new Runnable() {
			public void run() {
				int side = ((SungkaPlayerInfo)playerMap.get(new Integer(player))).getOrder();
				board[side][cup] = count;
				myView.processUpdateCup(player, cup);
			}
		});
	}

	/**
	 * Receives end of round message from the game board, and queues message
	 * for processing.
	 */
	public void endRound() {
		queueMessage(new Runnable() {
			public void run() { myView.processEndRound(); }
		});
	}

	/**
	 * Receives game over message from the game board, and queues message for
	 * processing.
	 * @param winner The player number of the winner of this game. Can equal
	 *               SungkaPlayer.GAMEOVER_TIE if tie game, or other negative
	 *               number if unexpected game finish.
	 */
	public void gameOver(final int winner) {
		queueMessage(new Runnable() {
			public void run() { myView.processGameOver(winner); }
		});
	}

	/**
	 * Receives chat message from the game board, and queues message for
	 * processing.
	 * @param msg The chat message.
	 * @param player The player the message was received from.
	 */
	public void chatMessage(final String msg, final int player) {
		queueMessage(new Runnable() {
			public void run() { myView.processChatMessage(msg, player); }
		});
	}

	/**
	 * Receives disconnect message from the game board, and queues message for\
	 * processing.
	 */
	public void disconnect() {
		queueMessage(new Runnable() {
			public void run() {
				if (!disconnect) {
					currentPlayer = -1;
					if (myRef != null) {
						myRef._release();
					}
					if (myBoard != null) {
						myBoard._release();
					}
					disconnect = true;
					myView.processDisconnect();
				}
			}
		});
	}

	/**
	 * Receives ping message from the Board.
	 * Calling this function is used simply to check the CORBA link, and does not require any
	 * processing other than the CORBA response. Body can be left empty.
	 */
	public void ping() {
	}

	/** Shuts down this player and the ORB associated with it. */
	protected void shutdown() {
		_orb().shutdown(true);
		_orb().destroy();
	}
}
