/*
 * Sungka v0.10.
 * SungkaPlayerView.java - Player View interface
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */

/**
 * The SungkaPlayerView interface specifies the methods required for a game
 * view to implement. The model (implemented by SungkaPlayerServant) calls
 * these methods to notify the view of changes in the game state.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public interface SungkaPlayerView {

	/**
	 * Logs a message to show to the user.
	 * @param s The actual message
	 * @param msgType A string representing the type of message.
	 *                Used to format the string output.
	 */
	public void log(String s, String msgType);

	/**
	 * Notifies view to reset game state when connecting to a new server.
	 * The model calls this after the model's state has been reset. A view
	 * must do any necessary processing to come as close to program startup
	 * state as possible.
	 */
	public void gameReset();

	/**
	 * Processes an update parameters message. Called by the model when game
	 * parameters (e.g. number of participants, number of cups) have changed.
	 */
	public void processUpdateParams();
	
	/**
	 * Processes an update board message. Called by model after the entire
	 * board contents have been changed.
	 */
	public void processUpdateBoard();

	/**
	 * Processes playerAdded message.
	 * @param s A SungkaPlayerInfo object with all the information of the added player.
	 */
	public void processPlayerAdded(SungkaPlayerInfo s);

	/**
	 * Processes playerRemoved message.
	 * @param s A SungkaPlayerInfo object with all the information of the removed player.
	 */
	public void processPlayerRemoved(SungkaPlayerInfo s);

	/**
	 * Processes startTurn message.
	 * @param player The player number of the player starting his turn.
	 */
	public void processStartTurn(int player);

	/**
	 * Processes endTurn message.
	 * @param player The player number of the player ending his turn.
	 */
	public void processEndTurn(int player);

	/**
	 * Processes playerMoved message.
	 * @param player The player number of the player making his move.
	 * @param cup The cup number the player is making his move from.
	 */
	public void processPlayerMoved(int player, int cup);

	/**
	 * Processes an update to a cup
	 * @param player The player number of the side receiving the stones.
	 * @param cup The cup number whose amount changed.
	 */
	public void processUpdateCup(int player, int cup);

	/** Processes endRound message.  */
	public void processEndRound();
	/**
	 * Processes gameOver message.
	 * @param winner The order number of the winner of this game. Negative if error occured,
	 *               equal to the number of players if there is a tie.
	 */
	public void processGameOver(int winner);

	/**
	 * Processes chatMessage message.
	 * @param msg The chat message.
	 * @param player The player the message was received from.
	 */
	public void processChatMessage(String msg, int player);

	/** 
	 * Processes disconnect message. Called by the model after it has already
	 * disconnected from the server.
	 */
	public void processDisconnect();
}
