/*
 * Sungka v0.10.
 * SungkaRegisterDialog.java - Game server settings dialog box
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Implements the settings dialog box for starting a new game server from
 * within the GUI View of the Sungka game.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class SungkaRegisterDialog extends JDialog implements SungkaConstants {
	/** True if OK button was pressed and data fields are valid. */
	private boolean inputOk = false;

	/**
	 * True if we want to register with the local Initial Naming Service.
	 * False if we want to use the Interoperable Naming Service.
	 */
	boolean localNameServer = false;

	/** The server we want to register this Sungka server with */
	String server = null;
	/** The port which the naming service is listening on. */
	int port = 0;

	/** The number of players to start the game board with. */
	int numPlayers = 0;
	/** The number of cups per player to start the board with. */
	int numCups = 0;
	/** The initial number of stone per cup to start the board with. */
	int numStones = 0;

	/** The text field to input the server to register with. */
	private final JTextField serverField = new JTextField("localhost", 20);
	/** The text field to input the port the naming service is listening on. */
	private final JTextField portField = new JTextField("1050", 4);

	/** The spinner for the number of players. */
	private final JSpinner playerSpinner =
		new JSpinner(new SpinnerNumberModel(2, 2, 99, 1));
	/** The spinner for the number of cups per player. */
	private final JSpinner cupSpinner =
		new JSpinner(new SpinnerNumberModel(7, 1, 99, 1));
	/** The spinner for the initial number of stones per cup. */
	private final JSpinner stoneSpinner =
		new JSpinner(new SpinnerNumberModel(7, 1, 99, 1));

	/** The radio button to register with the local Initial Naming Service. */
	private final JRadioButton localNameRadio;
	/** The radio button to register with the Interoperable Naming Service. */ 
	private final JRadioButton remoteNameRadio;

	/** The label for the server field. */
	private final JLabel serverLabel =
		new JLabel(resources.getString("GUI_server_server"));
	/** The label for the port field. */
	private final JLabel portLabel =
		new JLabel(resources.getString("GUI_server_port"));

	/** An array of preset game types that the player can choose from. */
	private static final SungkaGameType[] myGameTypes = { new SungkaGameType("Sungka", 2, 7, 7),
		new SungkaGameType("Mancala", 2, 6, 3),
		new SungkaGameType("Test", 2, 7, 1),
		new SungkaGameType("Multiplayer Sungka", 3, 7, 7),
		new SungkaGameType("Multiplayer Mancala", 3, 6, 3)
	};

	/**
	 * Constructs a new SungkaRegisterDialog
	 * @param parentWindow The parent window of this dialog
	 */
	SungkaRegisterDialog(Frame parentWindow) {
		super(parentWindow, resources.getString("GUI_server_title"), true);
		inputOk = false;
		JPanel content = new JPanel(new BorderLayout());

		// The Action for the OK button, which validates the input
		Action okAction = new AbstractAction(resources.getString("GUI_ok")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_K));
			}
			public void actionPerformed(ActionEvent e) {

				// extract input
				String portString = null;
				localNameServer = localNameRadio.isSelected();
				portString = portField.getText();
				if (!localNameServer) {
					server = serverField.getText();
				}

				numPlayers = ((Integer)playerSpinner.getValue()).intValue();
				numCups = ((Integer)cupSpinner.getValue()).intValue();
				numStones = ((Integer)stoneSpinner.getValue()).intValue();

				// validate input
				String errorMessage = "";

				// validate port
				try {
					port = Integer.parseInt(portString);
					portLabel.setForeground(defaultColor);
				} catch (NumberFormatException f) {
					errorMessage += "\n- " + resources.getString("GUI_server_badport");
					portLabel.setForeground(errorColor);
					portLabel.requestFocus();
				}

				// validate server
				if (!localNameServer) {
					if (server.equals("")) {
						errorMessage += "\n- " + resources.getString("GUI_server_badserver");
						serverLabel.setForeground(errorColor);
						serverField.requestFocus();
					} else {
						serverLabel.setForeground(defaultColor);
					}
				}

				if (errorMessage.equals("")) {
					inputOk = true;
					setVisible(false);
				} else {
					// reset to defaults, just in case
					localNameServer = false;
					server = null;
					port = 0;

					numPlayers = 0;
					numCups = 0;
					numStones = 0;

					// Show error dialog
					JOptionPane.showMessageDialog(SungkaRegisterDialog.this,
						resources.getString("GUI_server_pleasefix") + "\n" + errorMessage,
						resources.getString("GUI_server_errortitle"), JOptionPane.WARNING_MESSAGE);
				} // if
			} // actionPerformed
		}; // Abstractaction

		// The action for the Cancel button
		Action cancelAction = new AbstractAction(resources.getString("GUI_cancel")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
			}
			public void actionPerformed(ActionEvent e) {
				inputOk = false;
				setVisible(false);
			}
		}; // AbstractAction

		// The action for the local name server radio button. Disables server field.
		Action localAction = new AbstractAction(resources.getString("GUI_server_localserver")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_L));
			}
			public void actionPerformed(ActionEvent e) {
				serverField.setEnabled(false);
				serverLabel.setEnabled(false);
			}
		};

		// The action for the remote name server radio button. Enables server field.
		Action remoteAction = new AbstractAction(resources.getString("GUI_server_remoteserver")) {
			{
				putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_R));
			}
			public void actionPerformed(ActionEvent e) {
				serverField.setEnabled(true);
				serverLabel.setEnabled(true);
			}
		};


		// Game settings panel (at center/left)
		JPanel settingsPanel = new JPanel();
		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		settingsPanel.setLayout(gridBag);

		// Player label
		c.gridx = 1;
		c.anchor = GridBagConstraints.WEST;
		JLabel playerLabel = new JLabel(resources.getString("GUI_server_players"));
		playerLabel.setLabelFor(playerSpinner);
		gridBag.setConstraints(playerLabel, c);
		settingsPanel.add(playerLabel);

		// Player spinner
		c.gridx = GridBagConstraints.RELATIVE;
		((JTextField)((JSpinner.DefaultEditor)playerSpinner.getEditor()).getTextField()).setColumns(2);
		gridBag.setConstraints(playerSpinner, c);
		settingsPanel.add(playerSpinner);
		
		// Cup label
		c.gridx = 1;
		c.gridy = GridBagConstraints.RELATIVE;
		JLabel cupLabel = new JLabel(resources.getString("GUI_server_cups"));
		cupLabel.setLabelFor(cupSpinner);
		gridBag.setConstraints(cupLabel, c);
		settingsPanel.add(cupLabel);
		
		// Cup spinner
		c.gridx = 2;
		((JTextField)((JSpinner.DefaultEditor)cupSpinner.getEditor()).getTextField()).setColumns(2);
		gridBag.setConstraints(cupSpinner, c);
		settingsPanel.add(cupSpinner);

		// Stone label
		c.gridx = 1;
		c.gridy = GridBagConstraints.RELATIVE;
		JLabel stoneLabel = new JLabel(resources.getString("GUI_server_stones"));
		stoneLabel.setLabelFor(stoneSpinner);
		gridBag.setConstraints(stoneLabel, c);
		settingsPanel.add(stoneLabel);

		// Stone spinner
		c.gridx = 2;
		((JTextField)((JSpinner.DefaultEditor)stoneSpinner.getEditor()).getTextField()).setColumns(2);
		gridBag.setConstraints(stoneSpinner, c);
		settingsPanel.add(stoneSpinner);

		JSeparator sep = new JSeparator(SwingConstants.VERTICAL);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridy = 0;
		c.gridheight = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.VERTICAL;
		c.anchor = GridBagConstraints.NORTH;
		c.insets = new Insets(0, 10, 0, 10);
		gridBag.setConstraints(sep, c);
		settingsPanel.add(sep);

		c.gridx = 0;
		sep = new JSeparator(SwingConstants.VERTICAL);
		gridBag.setConstraints(sep, c);
		settingsPanel.add(sep);

		// Game type list (On left)
		final JList gameTypeList = new JList(myGameTypes);
		gameTypeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		gameTypeList.setVisibleRowCount(4);
		gameTypeList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				// We know we can only select one at a time, so getFirstIndex is enough
				SungkaGameType newType = (SungkaGameType)gameTypeList.getSelectedValue();
				if (newType != null) {
					playerSpinner.setValue(new Integer(newType.getNumPlayers()));
					cupSpinner.setValue(new Integer(newType.getNumCups()));
					stoneSpinner.setValue(new Integer(newType.getNumStones()));
				}
			}
		});

		playerSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				SungkaGameType curType = (SungkaGameType)gameTypeList.getSelectedValue();
				if (curType != null && ((Integer)playerSpinner.getValue()).intValue() != curType.getNumPlayers()) {
					gameTypeList.clearSelection();
				}
			}
		});
		cupSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				SungkaGameType curType = (SungkaGameType)gameTypeList.getSelectedValue();
				if (curType != null && ((Integer)cupSpinner.getValue()).intValue() != curType.getNumCups()) {
					gameTypeList.clearSelection();
				}
			}
		});
		stoneSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				SungkaGameType curType = (SungkaGameType)gameTypeList.getSelectedValue();
				if (curType != null && ((Integer)stoneSpinner.getValue()).intValue() != curType.getNumStones()) {
					gameTypeList.clearSelection();
				}
			}
		});

		JScrollPane gameTypeScrollPane = new JScrollPane(gameTypeList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		gameTypeScrollPane.getHorizontalScrollBar().setFocusable(false);
		gameTypeScrollPane.getVerticalScrollBar().setFocusable(false);

		// Name server settings panel (at right)
		JPanel namePanel = new JPanel();
		gridBag = new GridBagLayout();
		c = new GridBagConstraints();
		namePanel.setLayout(gridBag);

		ButtonGroup group = new ButtonGroup();
		localNameRadio = new JRadioButton(localAction);
		group.add(localNameRadio);
		remoteNameRadio = new JRadioButton(remoteAction);
		group.add(remoteNameRadio);
		localNameRadio.doClick();

		c.gridwidth = 2;
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		c.gridy = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(localNameRadio, c);
		namePanel.add(localNameRadio);

		c.gridy = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(remoteNameRadio, c);
		namePanel.add(remoteNameRadio);

		// Server label (Alt-S)
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		Insets oldInsets = c.insets;
		c.insets = new Insets(0, 20, 0, 0);
		serverLabel.setLabelFor(serverField);
		serverLabel.setDisplayedMnemonic(KeyEvent.VK_S);
		gridBag.setConstraints(serverLabel, c);
		namePanel.add(serverLabel);
		// Server textfield
		c.gridx = 1;
		c.insets = oldInsets;
		gridBag.setConstraints(serverField, c);
		namePanel.add(serverField);

		// Port label (Alt-P)
		portLabel.setLabelFor(portField);
		portLabel.setDisplayedMnemonic(KeyEvent.VK_P);
		c.gridx = 0;
		c.insets = new Insets(0, 20, 0, 0);
		c.gridy = GridBagConstraints.RELATIVE;
		gridBag.setConstraints(portLabel, c);
		namePanel.add(portLabel);
		// Port textfield
		c.gridx = 1;
		c.insets = oldInsets;
		gridBag.setConstraints(portField, c);
		namePanel.add(portField);

		// OK & Cancel Button panel (at bottom)
		JPanel okCancelPanel = new JPanel();
		okCancelPanel.setLayout(new BoxLayout(okCancelPanel, BoxLayout.X_AXIS));
		okCancelPanel.add(Box.createHorizontalGlue());
		
		// OK Button (Alt-K or Enter)
		JButton okButton = new JButton(okAction);
		okButton.setHorizontalTextPosition(AbstractButton.CENTER);
		okCancelPanel.add(okButton);

		okCancelPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		// Cancel Button (Alt-C or Escape)
		JButton cancelButton = new JButton(cancelAction);
		cancelButton.setHorizontalTextPosition(AbstractButton.CENTER);
		okCancelPanel.add(cancelButton);

		// Map escape and enter to cancel and okay's actions (NOT to the actual buttons)
		content.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancel");
		content.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "ok");
		content.getActionMap().put("ok", okAction);
		content.getActionMap().put("cancel", cancelAction);

		// Add components
		content.add(gameTypeScrollPane, BorderLayout.WEST);
		content.add(settingsPanel, BorderLayout.CENTER);
		content.add(namePanel, BorderLayout.EAST);
		content.add(okCancelPanel, BorderLayout.SOUTH);
		content.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		setContentPane(content);

		setResizable(false);
		pack();
		// setLocationRelativeTo(parentWindow);

		addWindowListener(new WindowAdapter() {
			private boolean gotFocus = false;

			public void windowClosing(WindowEvent we) {
				inputOk = false;
			}

			public void windowGainedFocus(WindowEvent we) {
				// Once window gets focus, set initial focus
				if (!gotFocus) {
					// nameField.requestFocus();
					gotFocus = true;
				}
			} // windowGainedFocus
		}); // WindowAdapter
	} // SungkaRegisterDialog

	/**
	 * Tells whether the input was valid. (e.g. the user hit the OK button.)
	 * @return True if the input was valid
	 */
	public boolean inputOk() {
		return inputOk;
	}
} // class SungkaRegisterDialog

