/*
 * Sungka v0.10.
 * SungkaRegisterServant.java - Game Board class
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import org.omg.CORBA.ORB;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.IntHolder;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POA;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.io.*;

/**
 * SungkaRegisterServant implements the playing board side of the game.
 * 
 * This class only exposes the SungkaRegister interface, which only
 * allows a method to join the game. However, it contains all the game logic
 * within it.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class SungkaRegisterServant extends SungkaRegisterPOA
	implements Runnable {

	/** The actual playing board. */
	protected int[][] board;

	/**
	 * List of players. The index of a player in this ArrayList determines
	 * their player number.
	 */
	protected ArrayList players = new ArrayList();

	/**
	 * An array of participants. The index of a player in this array
	 * determines the order that players take turns in.
	 */
	protected SungkaBoardServant[] participants;

	/**
	 * A list of messages to be processed. Currently used only for player
	 * removal.
	 */
	protected List messages = Collections.synchronizedList(new LinkedList());

	/** A PrintWriter which we send all log messages to. */
	protected PrintWriter log = new PrintWriter(System.out, true);

	/** The number of participants in this game. */
	protected final int numParticipants;
	/** The number of cups per participant in this game. */
	protected final int numCups;
	/** The initial number of stones per cup in this game. */
	protected final int numStones;

	/** The current participant who we are waiting on to make a move. */
	protected int currentParticipant;

	/** The number of participants ready. */
	protected volatile int numReady;

	/** The current number of participants registered. */
	protected int currentNumParticipants = 0;

	/** True if the game has started already. */
	protected volatile boolean gameStarted;

	/**
	 * A semaphore for shutting down the game. When set to true, the main
	 * event loop will stop the game, notify all players of the disconnect,
	 * and shutdown.
	 */
	protected volatile boolean doShutdown = false;

	/** The current state of the server. */
	protected volatile int currentState;

	/** State constant for a server that is shutdown/not running. */
	public static final int SERVER_SHUTDOWN = 0;
	/** State constant for a server that is in the process of starting. */
	public static final int SERVER_STARTING = 1;
	/** State constant for a server that is running. */
	public static final int SERVER_RUNNING = 2;
	/** State constant for a server that is in the process of shutting down. */
	public static final int SERVER_SHUTTING_DOWN = 3;

	/** The Thread we are running in. Used for blocking until shutdown. */
	private Thread myThread;

	/** The server view associated with this instance. */
	private SungkaRegisterView myView = null;

	/** A Java reflection Method of the SungkaPlayer.playerAdded method */
	private static final Method PLAYER_ADDED =
		getMethod("playerAdded", new Class[]{ String.class, Integer.TYPE,
			Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.playerRemoved method */
	private static final Method PLAYER_REMOVED =
		getMethod("playerRemoved", new Class[]{ Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.startTurn method */
	private static final Method START_TURN =
		getMethod("startTurn", new Class[]{ Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.endTurn method */
	private static final Method END_TURN =
		getMethod("endTurn", new Class[]{ Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.playerMoved method */
	private static final Method PLAYER_MOVED =
		getMethod("playerMoved", new Class[]{ Integer.TYPE, Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.addToCup method */
	private static final Method ADD_TO_CUP =
		getMethod("addToCup", new Class[]{ Integer.TYPE, Integer.TYPE,
			Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.setCup method */
	private static final Method SET_CUP =
		getMethod("setCup", new Class[]{ Integer.TYPE, Integer.TYPE,
			Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.endRound method */
	private static final Method END_ROUND =
		getMethod("endRound", new Class[0]);
	/** A Java reflection Method of the SungkaPlayer.gameOver method */
	private static final Method GAME_OVER =
		getMethod("gameOver", new Class[]{ Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.chatMessage method */
	private static final Method CHAT_MESSAGE =
		getMethod("chatMessage", new Class[]{ String.class, Integer.TYPE });
	/** A Java reflection Method of the SungkaPlayer.ping method */
	private static final Method PING =
		getMethod("ping", new Class[0]);
	/** A Java reflection Method of the SungkaPlayer.disconnect method */
	private static final Method DISCONNECT =
		getMethod("disconnect", new Class[0]);
	/** A Java reflection Method of the SungkaPlayer.updateBoard method */
	private static final Method UPDATE_BOARD =
		getMethod("updateBoard", new Class[]{ new int[0][0].getClass() });
	/** An empty array, for tellPlayer methods which take no arguments. */
	private static final Object[] NO_ARGS = new Object[0];

	/**
	 * Gets the method of the SungkaPlayer class with the given name and
	 * argument types.
	 * @param name The name of the method
	 * @param argTypes The argument types to this method, as specified by Java
	 *                 reflection
	 * @return A Method object that reflects the given method in the
	 *         SungkaPlayer class
	 */
	private static Method getMethod(String name, Class[] argTypes) {
		try {
			return SungkaPlayer.class.getMethod(name, argTypes);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Creates a new Sungka Board.
	 *
	 * @param numParticipants The number of participants in this game.
	 * @param numCups The number of normal cups per participant.
	 * @param numStones The number of stones per cub to begin with.
	 */
	public SungkaRegisterServant(int numParticipants, int numCups, int numStones) {
		setState(SERVER_STARTING);
		this.numCups = numCups;
		this.numParticipants = numParticipants;
		this.numStones = numStones;

		// We add an extra cup to each row for the home cup.
		board = new int[numParticipants][numCups + 1];
		participants = new SungkaBoardServant[numParticipants];

		resetGame();
	}

	/**
	 * Sets the current state of the game server, and notifies all views.
	 * @param newState The new state that the server is in.
	 */
	protected void setState(int newState) {
		currentState = newState;
		if (myView != null) {
			myView.updateState(newState);
		}
	}
	
	/**
	 * Associates a view with this game server.
	 * NOTE: SungkaRegisterServant currently only supports one view at a time.
	 *       If there is an existing view and another one is registered, the
	 *       old one will be replaced.
	 * @param view The view to associate with this game server.
	 */
	public void addView(SungkaRegisterView view) {
		myView = view;
		log = myView.getLogPrintWriter();
		myView.updateState(currentState);
	}

	/** Resets the game back to its initial state. */
	protected void resetGame() {
		synchronized (board) {
			// Initialize all the cups for all participants.
			for (int i = 0; i < numParticipants; i++) {
				board[i][numCups] = 0;
				for (int j = 0; j < numCups; j++) {
					board[i][j] = numStones;
				}
			}
			currentParticipant = -1;
			numReady = 0;
			gameStarted = false;
		}

		// Make all players not ready
		synchronized (players) {
			Iterator i = players.iterator();
			while (i.hasNext()) {
				SungkaBoardServant b = (SungkaBoardServant)i.next();
				if (b != null) {
					synchronized (b) {
						b.isReady = false;
						// FIXME need to reset!
					}
				}
			}
		}
	}

	/**
	 * Called by a new player to join this game.
	 *
	 * @param p CORBA reference to remote new player object
	 * @param name The name for this new player
	 * @param observer True if this player wants to just observe.
	 * @param playerNum Returns the player number for the new player.
	 * @return A reference to a SungkaBoard CORBA object that the player can
	 *         use to communicate moves, or null if player not allowed to join.
	 */
	public SungkaBoard joinGame(SungkaPlayer p, String name, boolean observer,
			IntHolder playerNum) {
		try {
			ArrayList playersCopy;
			int myParticipantNum = -1;
			SungkaBoardServant boardServant;
			int myNum;

			// Synchronized on board to prevent a participant from making a
			// turn so we can keep board in a consistent state for new player.
			synchronized (board) {
				if (!observer) {
					// Check if we're full. If not, reserve our slot in game
					synchronized (participants) {
						if (currentNumParticipants >= numParticipants) {
							// Need dummy values to avoid causing an exception.
							// Can't be null.
							playerNum.value = -1;
							return null;
						}
						currentNumParticipants++;
					}
				}

				synchronized (players) {
					// Make temporary copy of players for telling incoming
					// player of incoming players. We clone it to reduce the
					// amount of time the players array is locked
					playersCopy = (ArrayList)players.clone();

					// Find an unused player number. If players have left the
					// game before, we reuse their player number.
					myNum = players.indexOf(null);
					if (myNum < 0) {
						myNum = players.size();
					}

					// So the object creation is within the locked section.
					// I'll deal with the inefficiency for now.
					boardServant =
						new SungkaBoardServant(this, p, myNum, name, _poa());

					// Add us to the players array. After this, the new player
					// will get game updates
					if (myNum == players.size()) {
						players.add(boardServant);
					} else {
						players.set(myNum, boardServant);
					}
				} // synchronized (players)

				if (!observer) {
					synchronized (participants) {
						// Find null participant slot. Already guaranteed our
						// space, no need to error check
						for (myParticipantNum = 0;
							 myParticipantNum < participants.length &&
							 participants[myParticipantNum] != null;
							 myParticipantNum++);
						participants[myParticipantNum] = boardServant;
					} // synchronized (participants)
				}

				// set return values
				playerNum.value = myNum;
				requestParams(boardServant);
				requestBoard(boardServant);
			} // synchronized (board)
			log.println("New Player! #" + myNum + "/participant #" +
					myParticipantNum);

			// Tell new player of existing players' names
			// In theory, these can be done in another thread
			Iterator i = playersCopy.iterator();
			while (i.hasNext()) {
				SungkaBoardServant b = (SungkaBoardServant)i.next();
				if (b == null) {
					continue;
				}
				int othParticipantNum = getParticipantNum(b);

				// Check if person is still active. If not, don't tell of added
				// player. Prevents "remove before add" race condition
				synchronized (b) {
					if (b.playerRef != null) {
						p.playerAdded(b.name, b.player,
								(othParticipantNum < 0 ? -1 :
								 ((myParticipantNum < 0 ? 0 : myParticipantNum)
								  + othParticipantNum) % numParticipants));
					}
				} // synchronized (b)
			} // while

			// Tell all players of the new player
			tellPlayers(PLAYER_ADDED, new PlayerAddedArgsGen(boardServant));

			// Return reference to our board reference
			return boardServant.href;
		} catch (Exception e) {
			e.printStackTrace(log);
			return null;
		}
	} // joinGame

	/**
	 * Process a request to send the game parameters to a player.
	 * @param s SungkaBoardServant of player who is asking for the parameters .
	 */
	public void requestParams(SungkaBoardServant s) {
		int[] returnParams = getParams();
		synchronized (s) {
			// FIXME catch CORBA exceptions
			if (s.playerRef != null) {
				s.playerRef.updateParams(returnParams);
			}
		}
	}

	/**
	 * Get the parameters of this Sungka game.
	 * @return An array with the parameters of this game.
	 */
	protected int[] getParams() {
		int[] params = new int[3];
		params[SungkaBoard.PLAYER_COUNT] = numParticipants;
		params[SungkaBoard.CUP_COUNT] = numCups;
		synchronized (board) {
			params[SungkaBoard.CURRENT_PLAYER] = currentParticipant;
		}
		return params;
	}

	/**
	 * Get the player number of given participant.
	 * @param participant The participant number of a player.
	 * @return The corresponding player number.
	 */
	protected int getPlayerNum(int participant) {
		synchronized (participants) {
			return participants[participant].player;
		}
	}

	/**
	 * Get the participant number of given player from ORB reference.
	 * @param s The servant corresponding to a player.
	 * @return The corresponding participant number.
	 */
	protected int getParticipantNum(SungkaBoardServant s) {
		// we iterate since we assume that the cost of iteration is not too
		// high may need to rethink if we change our data structure
		int i = 0;
		synchronized (participants) {
			for (; i < participants.length && participants[i] != s; i++);
			return (i >= participants.length ? -1 : i);
		}
	}

	/**
	 * Get the participant number of given player from the player number.
	 * @param player The player number of a player.
	 * @return The corresponding participant number.
	 */
	protected int getParticipantNum(int player) {
		synchronized (players) {
			return getParticipantNum((SungkaBoardServant)players.get(player));
		}
	}

	/**
	 * Process a request to send a board update to a player.
	 * @param s SungkaBoardServant of player who is asking for the board.
	 */
	protected void requestBoard(SungkaBoardServant s) {
		synchronized (board) {
			int[][] returnBoard = getBoard(s);
			synchronized (s) {
				// FIXME catch CORBA exceptions
				if (s.playerRef != null) {
					s.playerRef.updateBoard(returnBoard);
				}
			}
		}
	}

	/**
	 * Returns the current board layout, from point of view of given player.
	 * @param s SungkaPlayerServant of player who is asking for the board.
	 * @return The contents of the board, rotated to match view of the player.
	 */
	protected int[][] getBoard(SungkaBoardServant s) {
		int[][] returnBoard = new int[numParticipants][numCups + 1];

		int start = getParticipantNum(s);
		if (start < 0) {
			start = 0;
		}

		int curParticipant = start;
		int i = 0;
		synchronized (board) {
			do {
				System.arraycopy(board[curParticipant], 0, returnBoard[i], 0,
						numCups + 1);
				i++;
			} while ((curParticipant = nextParticipant(curParticipant)) != start);
			return returnBoard;
		}
	}

	/**
	 * Registers this server with the name server with either the local Initial
	 * Naming Service or remote Interoperable Naming Service.
	 * @param remote If true, will try to register this server with the
	 *               Interoperable Name Service. Otherwise will register with
	 *               the local Initial Naming Service. For remote registration,
	 *               the ORB must have its reference to NameService already set
	 *               before calling the method.
	 * @return true if registration was successful, false if an error occured.
	 */
	public boolean registerServer(boolean remote) {
		try {
			POA rootpoa = POAHelper.narrow(_orb().
					resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
			if (remote) {
				NamingContextExt name = NamingContextExtHelper.narrow(_orb().
						string_to_object("corbaname:rir:#"));
						// using the real resolve_initial_references causes the
						// game to freeze if you have multiple ORBs in one
						// process, since ORBs actually share data structures.
						// We use the corbaname:rir bug for now FIXME
						// resolve_initial_references("NameService"));
				if (name == null) {
					return false;
				}
				name.rebind(name.to_name("Sungka"),
						rootpoa.servant_to_reference(this));
			} else {
				((com.sun.corba.se.internal.Interceptors.PIORB)_orb()).
					register_initial_reference("Sungka",
							rootpoa.servant_to_reference(this));
			}
			return true;
		} catch (Exception e) {
			e.printStackTrace(log);
			return false;
		}
	}

	/**
	 * Starts this game. Perfect for starting the game in a new thread.
	 */
	public void run() {
		myThread = Thread.currentThread();
		log.println("SungkaBoard ready and waiting ...");

		setState(SERVER_RUNNING);
		while (!doShutdown) {
			// Process any messages before starting next participant.
			while (messages.size() > 0) {
				Runnable r = (Runnable)messages.remove(0);
				r.run();
			}

			if (!gameStarted) {
				if (numReady == numParticipants) {
					// Start playing!
					synchronized (board) {
						currentParticipant = 0;
						tellPlayers(UPDATE_BOARD, new UpdateBoardArgsGen());
					}
					gameStarted = true;
					startTurn();
				}
			} else {
				// Exit game & determine winner after processing last messages
				if (gameOver()) {
					// prevents calls to readyToPlay from notify of gameOver
					// to actual reset
					synchronized (players) {
						determineWinner();
						resetGame();
					}
				}
			}

			synchronized (this) {
				try {
					wait(10000);
				} catch (InterruptedException e) {
					e.printStackTrace(log);
					doShutdown = true;
				}
			} // synchronized (this)

			// ping all players to detect dead players
			tellPlayers(PING, NO_ARGS);
		} // while
		setState(SERVER_SHUTTING_DOWN);
		tellPlayers(DISCONNECT, NO_ARGS);
		log.println("SungkaBoard Exiting ...");
		setState(SERVER_SHUTDOWN);
		log.close();
	}

	/** Determines the winner of the game, and notifies all players. */
	protected void determineWinner() {
		int winner = -1;
		int maxValue = -1;

		// Detect premature end (i.e. lost connection to a participant)
		if (numParticipants == numReady) {
			// Determine the winner
			// synch doesn't seem to be strictly necessary (since no one else
			// should be changing the board), but might as well lock it
			synchronized (board) {
				for (int i = 0; i < numParticipants; i++) {
					if (board[i][numCups] > maxValue) {
						maxValue = board[i][numCups];
						winner = i;
					} else if (board[i][numCups] == maxValue) {
						// Detected a tie!
						winner = SungkaPlayer.GAMEOVER_TIE;
					}
				}
			} // synchronized (board)
		}
		if (winner >= 0) {
			winner = getPlayerNum(winner);
		}
		tellPlayers(GAME_OVER, new Object[]{ new Integer(winner) });
	}

	/**
	 * Starts game from the command line.
	 * @param args Command line arguments to the game. See documentation.
	 */
	public static void main(String [] args) {
		// process command line arguments
		boolean remote = false;
		String host = null;
		int port = 0;
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-remote")) {
				remote = true;
			}
		}

		try {
			// create and initialize the ORB
			ORB orb = ORB.init(args, null);
			SungkaRegisterServant sungkaServant =
				new SungkaRegisterServant(2, 7, 1);
			((org.omg.CORBA_2_3.ORB)orb).set_delegate(sungkaServant);
			if (!sungkaServant.registerServer(remote)) {
				System.err.println("ERROR: Could not register server!");
				System.exit(1);
			}

			sungkaServant.run();
			sungkaServant._orb().shutdown(true);
			sungkaServant._orb().destroy();
		} catch (Exception e) {
			System.err.println("ERROR: " + e);
			e.printStackTrace();
		}
	}

	/**
	 * Removes a player from the game.
	 * @param b SungkaBoardServant to be removed
	 */
	protected void removePlayer(SungkaBoardServant b) {
		// Remove player from players ArrayList, and replace with a null
		synchronized (players) {
			players.set(b.player, null);
		}

		// Null out our references to mark as inactive, to "tell" all cloned
		// lists, and release resources
		SungkaPlayer playerRef;
		SungkaBoard href;
		synchronized (b) {
			playerRef = b.playerRef;
			href = b.href;
			b.playerRef = null;
			b.href = null;
			if (b.isReady) {
				b.isReady = false;
				numReady--;
			}
		}
		playerRef._release();
		href._release();

		final int player = b.player;
		// Detect if the player is a participant
		int participant = getParticipantNum(b);
		log.println("Removing player: " + player + " --> " + participant);
		if (participant >= 0) {
			synchronized (participants) {
				currentNumParticipants--;
				participants[participant] = null;
			}
		}

		// Queue up player removed message
		messages.add(new Runnable() {
			public void run() {
				tellPlayers(PLAYER_REMOVED, new Object[]{ new Integer(player)});
			}
		});
		synchronized (this) {
			notifyAll();
		}
	}

	/**
	 * Call the given method with the given arguments for every player object.
	 * @param m A method in the SungkaPlayer class
	 * @param args Arguments for this method, as specified in Java reflection.
	 */
	protected void tellPlayers(Method m, Object[] args) {
		tellPlayers(m, new DefaultArgsGen(args));
	}

	/**
	 * Call the given method with the arguments created from the given argument
	 * generator for every player object.
	 * @param m A method in the SungkaPlayer class
	 * @param argGen Argument generator that produces arguments for the method.
	 */
	protected void tellPlayers(Method m, SungkaArgsGen argGen) {
		ArrayList playersCopy;
		synchronized (players) {
			playersCopy = (ArrayList)players.clone();
		}

		Iterator i = playersCopy.iterator();
		while (i.hasNext()) {
			SungkaBoardServant boardServant = (SungkaBoardServant)i.next();
			if (boardServant == null) {
				continue;
			}

			synchronized (boardServant) {
				SungkaPlayer playerRef = boardServant.playerRef;
				if (playerRef == null) {
					continue;
				}

				try {
					m.invoke(playerRef, argGen.getArgs(boardServant));
				} catch (IllegalAccessException e) {
					e.printStackTrace(log);
					doShutdown = true;
				} catch (IllegalArgumentException e) {
					e.printStackTrace(log);
					doShutdown = true;
				} catch (InvocationTargetException e) {
					try {
						throw e.getCause();
					} catch (COMM_FAILURE f) {
						removePlayer(boardServant);
					} catch (Throwable f) {
						f.printStackTrace(log);
						doShutdown = true;
					} // try
				} // try
			} // synchronized (boardServant)
		} // while
	}

	/**
	 * Call given method with the given arguments for every player object.
	 * @param func Name of the function to be called
	 * @param argTypes An array of Class objects representing the types of each 
	 *                 argument, as specified in Java reflection
	 * @param args Arguments for this method, as specified in Java reflection.
	 */
	protected void tellPlayers(String func, Class[] argTypes, Object[] args) {
		try {
			tellPlayers(SungkaPlayer.class.getMethod(func, argTypes), args);
		} catch (NoSuchMethodException e) {
			e.printStackTrace(log);
			doShutdown = true;
		}
	} // tellPlayers

	/**
	 * Check if this Sungka game is over (i.e. nobody has any legal moves).
	 * @return True if no legal moves left for any participant.
	 */
	protected boolean gameOver() {
		boolean allEmpty = true;
		// If we don't have enough participants, the game is over.
		if (numReady == numParticipants) {
			// Check that all participants have no moves left
			for (int i = 0; i < 2 && allEmpty; i++) {
				allEmpty = !hasMove(i);
			}
		}
		return allEmpty;
	}

	/**
	 * Check if the given participant has any legal moves left.
	 * @param participant The participant number.
	 * @return True if given participant has any moves left, false otherwise.
	 */
	protected boolean hasMove(int participant) {
		boolean allEmpty = true;
		// Check that all cups on this participant's side are empty.
		synchronized (board) {
			for (int j = 0; j < numCups && allEmpty; j++) {
				allEmpty = board[participant][j] == 0;
			}
		}
		return !allEmpty;
	}

	/**
	 * Returns the next participant to move after the given participant.
	 * @param participant The participant number.
	 * @return The participant who follows the specified participant.
	 */
	protected final int nextParticipant(int participant) {
		return (++participant >= numParticipants ? 0 : participant);
	}

	/**
	 * Set given cup to have the given number of stones, and tell all players.
	 *
	 * Assumption: Caller owns the lock to board
	 *
	 * @param side Which participant's side the cup is on
	 * @param cup The number of the cup to set
	 * @param number The number of stones to put in this cup
	 */
	protected void setCup(int side, int cup, int number) {
		board[side][cup] = number;
		tellPlayers(SET_CUP, arrayPack(getPlayerNum(side), cup, number));
	}

	/**
	 * Packs 3 int's into Integer objects, and packs them into an Object array.
	 * @param a The first int in the array
	 * @param b The second int in the array
	 * @param c The third int in the array
	 * @return An Object array containing the 3 int's in Integer wrappers.
	 */
	public static Object[] arrayPack(int a, int b, int c) {
		return new Object[]{ new Integer(a), new Integer(b), new Integer(c) };
	}

	/**
	 * Add the given number of stones to given cup, and tell all players.
	 *
	 * Assumption: Caller owns the lock to board
	 *
	 * @param side Which player's side the cup is on
	 * @param cup The number of the cup to add to
	 * @param number The number of stones to add to this cup
	 */
	protected void addToCup(int side, int cup, int number) {
		board[side][cup] += number;
		tellPlayers(ADD_TO_CUP, arrayPack(getPlayerNum(side), cup, number));
	}

	/**
	 * Called by a participant to tell what move to make.
	 * @param player The player number of the participant making a move.
	 * @param cup The cup number the participant is making a move in.
	 * @return True if move is legal and accepted, false if move is illegal.
	 */
	protected boolean makeMove(int player, int cup) {
		int participant = getParticipantNum(player);

		// Check that it's this participants turn
		synchronized (board) {
			if (participant != currentParticipant) {
				return false;
			}
		}

		// Check that the given cup is not empty
		// FIXME need to check if this participant was ready
		int currentSide = participant;
		if (!isValidMove(currentSide, cup)) {
			return false;
		}

		// Tell all players of the move
		tellPlayers(PLAYER_MOVED, new Object[]{ new Integer(player),
			new Integer(cup) });

		// Calculate the changes to the board
		synchronized (board) {
			while (currentSide == participant && cup < numCups &&
					board[currentSide][cup] > 0) {
				// Empty cup and start distributing its contents
				int numLeft = board[currentSide][cup];
				setCup(currentSide, cup, 0);
				while (numLeft > 0) {
					while (++cup < (numCups +
								(currentSide == participant ? 1 : 0)) &&
							numLeft-- > 0) {
						addToCup(currentSide, cup, 1);
					}

					// Get the last cup we deposited into
					cup--;

					if (numLeft > 0) {
						// If we're here, we've hit the end of the row.
						// No need to check again.
						currentSide = nextParticipant(currentSide);
						cup = -1;
					} else if (currentSide == participant &&
							cup < numCups && board[currentSide][cup] == 1) {
						// 2nd check assures last piece not deposited in home
						int nextSide = nextParticipant(currentSide);
						addToCup(currentSide, numCups, board[currentSide][cup] +
								board[nextSide][numCups - cup - 1]);
						setCup(currentSide, cup, 0);
						setCup(nextSide, numCups - cup - 1, 0);
						currentSide = nextSide;
					} // if
				} // while
			} // while
			// Tell each player when we've run out of stones
			tellPlayers(END_ROUND, NO_ARGS);
		} // synchronized (board)

		// Check if we ended up on our own side or if we have no moves left
		// No need to relock currentParticipant yet - no chance of reaching this
		// point unless we had it locked correctly.
		if (!(currentSide == currentParticipant && hasMove(currentParticipant))) {
			tellPlayers(END_TURN, new Object[]{ new Integer(player) });
			if (gameOver()) {
				synchronized (this) {
					notifyAll();
				}
			} else {
				// Lock board; prevents race condition as currentParticipant changes
				synchronized (board) {
					while (!hasMove(currentParticipant = nextParticipant(currentParticipant)));
				}
			}
		} // if

		startTurn();
		return true;
	} // makeMove

	/**
	 * Sends a start turn message for the current participant to all players.
	 */
	protected void startTurn() {
		if (!gameOver()) {
			tellPlayers(START_TURN,
					new Object[]{ new Integer(getPlayerNum(currentParticipant)) });
		}
	}

	/**
	 * Checks if the move by the given participant is legal.
	 *
	 * @param participant The participant number.
	 * @param cup The cup where this participant plans to move.
	 * @return True if the move is valid, false otherwise.
	 */
	protected boolean isValidMove(int participant, int cup) {
		synchronized (board) {
			return cup >= 0 && cup < numCups && board[participant][cup] != 0;
		}
	}

	/**
	 * Tells board that player is ready. Game starts when all participants ready
	 * @param player The player number of the player ready to start.
	 */
	protected void readyToPlay(SungkaBoardServant s) {
		if (getParticipantNum(s) >= 0) {
			numReady++;
			log.println("Ready count: " + numReady);
			synchronized (this) {
				notifyAll();
			}
		}
	}

	/**
	 * Sends a broadcast chat message from the given player to all players
	 * @param player The player number of the player sending the message.
	 * @param msg The chat message.
	 */
	protected void sendChatMessage(int player, String msg) {
		tellPlayers(CHAT_MESSAGE, new Object[]{ msg, new Integer(player) });
	}

	/**
	 * Tells board that a player is leaving the game.
	 * @param player The player number of the player exiting game.
	 */
	protected void exitGame(SungkaBoardServant b) {
		removePlayer(b);
	}

	/**
	 * Requests that the server shutdown.
	 * @param wait True if we should block until server is shut down.
	 */
	public void shutdown(boolean wait) {
		doShutdown = true;
		synchronized (this) {
			notifyAll();
		}
		try {
			if (wait) {
				myThread.join();
			}
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Prints a message to the log.
	 * @param msg The log message
	 */
	protected void log(String msg) {
		log.println(msg);
	}

	/**
	 * Generates arguments for the playerAdded broadcast message.
	 */
	private class PlayerAddedArgsGen implements SungkaArgsGen {
		/** The SungkaBoardServant for the player being added. */
		private SungkaBoardServant arg;

		/**
		 * Creates a new PlayerAdded argument generator.
		 * @param arg The SungkaBoardServant for the player being added.
		 */
		PlayerAddedArgsGen(SungkaBoardServant arg) {
			this.arg = arg;
		}

		/**
		 * Returns the arguments associated with the playerAdded method.
		 * @param s The SungkaBoardServant object associated with a player.
		 * @return An Object array containing the arguments for updateBoard,
		 *         customized for this player. In this case, we get an array
		 *         with the order number rotated to the proper position.
		 */
		public Object[] getArgs(SungkaBoardServant s) {
			int participant = getParticipantNum(s);
			int othParticipantNum = getParticipantNum(arg);
			return new Object[]{ arg.name, new Integer(arg.player),
					new Integer((participant < 0 ? -1 :
								(participant +
								 (othParticipantNum < 0 ? 0 :
								  othParticipantNum)) % numParticipants)) };
		}
	}

	/**
	 * Generates arguments for the updateBoard broadcast message.
	 */
	private class UpdateBoardArgsGen implements SungkaArgsGen {
		/**
		 * Returns the arguments associated with updateBoard for this player.
		 * @param s The SungkaBoardServant object associated with a player.
		 * @return An Object array containing the arguments for updateBoard,
		 *         customized for this player. In this case, we get an array
		 *         with one element, the board contents.
		 */
		public Object[] getArgs(SungkaBoardServant s) {
			synchronized (board) {
				return new Object[]{ getBoard(s) };
			}
		}
	}
}
