/*
 * Sungka v0.10.
 * TextSungkaPlayerView.java - Player Text-Only View class
 * Copyright (c) 2003 Michael Barrientos. All rights reserved.
 */
import java.io.*;
import java.text.MessageFormat;
import org.omg.CORBA.ORB;
import java.util.Random;

/**
 * TextSungkaPlayerView implements a text only view of the player side of the game.
 *
 * @author Michael Barrientos
 * @version 0.10, 3/4/2003
 */
public class TextSungkaPlayerView extends AbstractSungkaPlayerView {
	/**
	 * A BufferedReader to read player keyboard input. If multiple players are
	 * sharing a display, all instances share this BufferedReader.
	 */
	protected static BufferedReader input =
		new BufferedReader(new InputStreamReader(System.in));

	/** 
	 * Tells if we are being played locally. Used when multiple SungkaPlayer's are
	 * sharing a single terminal for output, so that each players output does not
	 * conflict with one another.
	 */
	protected boolean isLocal;

	/**
	 * Combined with synchronizing on the class object, this boolean is used to
	 * lock the display when playing a local game. Prevents race conditions
	 * when printing to the screen right when the current player is changing.
	 */
	protected static boolean locked = false;

	/**
	 * Determines if the previous move was made by this player. Used to prevent
	 * 2 players that are sharing a display from displaying a message twice.
	 * By convention, the player whose turn is next will display the message.
	 */
	protected boolean lastMyTurn = false;

	/**
	 * Creates a new text-only view of the Sungka game.
	 * @param servant The servant that this view will be associated with.
	 * @param isLocal True if display is being shared with other text-only views
	 */
	public TextSungkaPlayerView(SungkaPlayerServant model, boolean isLocal) {
		super(model);
		this.isLocal = isLocal;
	}

	/**
	 * Logs a message to show to the user.
	 * @param s The actual message
	 * @param msgType A string representing the type of message.
	 *                Used to format the string output.
	 */
	public void log(String s, String msgType) {
		if (!isLocal || myModel.myTurn() || msgType.equals(LOG_ERROR)) {
			System.out.println(s);
		}
	}

	/**
	 * Processes startTurn message.
	 * @param player The player number of the player starting his turn.
	 */
	public void processStartTurn(int player) {
		if (myModel.myTurn()) {
			// It's my turn! Ask the player for his next move
			if (isLocal && !lastMyTurn) {
				// Lock the display if it's my turn and it wasn't my turn before.
				lockDisplay();
			}
			int myMove = -1;
			while (myMove < 0) {
				log(formatString(resources.getString("text_startturn"),
							myModel.getPlayerName(myModel.getCurrentPlayer())),
						LOG_MOVE);
				displayBoard();
				System.out.print(resources.getString("text_yourmove") + " ");
				try {
					// Input the space to move into
					myMove = Integer.parseInt(input.readLine()) - 1;
				} catch (NumberFormatException e) {
					log(resources.getString("text_badcup"), LOG_GAMEERROR);
					continue;
				} catch (IOException e) {
					e.printStackTrace();
					System.exit(1);
				}
				if (!myModel.myBoard.makeMove(myMove)) {
					myMove = -1;
					log(resources.getString("badmove"), LOG_GAMEERROR);
				}
			}
		} else if (!isLocal) {
			// Just display whose turn it is, and sleep until the next message.
			log(formatString(resources.getString("waitturn"),
						myModel.getPlayerName(myModel.getCurrentPlayer())),
					LOG_MOVE);
			displayBoard();
		} else if (lastMyTurn) {
			// We're sure we are local at this point.
			// If it's no longer our turn, release the display.
			unlockDisplay();
		}
		lastMyTurn = myModel.myTurn();
	}

	/**
	 * Processes end of round message.
	 */
	public void processEndRound() {
		if (!isLocal || myModel.myTurn()) {
			// Only one local player needs to print the board.
			System.out.println(resources.getString("text_currentboard"));
			displayBoard();
		}
	}

	/**
	 * Displays a text representation of this board.
	 */
	protected void displayBoard() {
		int[][] board = myModel.getBoard();
		int numCups = myModel.getNumCups();
		String format = "     |     |     |     |     |     |     |     |     ";
		String separa = "-----+-----+-----+-----+-----+-----+-----+-----+-----";
		StringBuffer s = new StringBuffer(format);
		for (int i = 1, j = 8 ; i <= numCups; i++, j += 6) {
			s.replace(j, j + 1, i + "");
		}
		System.out.println(s);
		System.out.println(separa);
		s.replace(0, s.length(), format);
		for (int i = numCups, j = 4; i >= 0; i--, j += 6) {
			s.replace(j - 1 - (board[1][i] >= 10 ? 1 : 0) - (board[1][i] >= 100 ? 1 : 0), j, board[1][i] + "");
		}
		System.out.println(s);
		s.replace(0, s.length(), format);
		for (int i = 0, j = 10; i <= numCups; i++, j += 6) {
			s.replace(j - 1 - (board[0][i] >= 10 ? 1 : 0) - (board[0][i] >= 100 ? 1 : 0), j, board[0][i] + "");
		}
		System.out.println(s);
	}

	/**
	 * Processes gameOver message.
	 * @param winner The order number of the winner of this game. Can equal
	 *               SungkaRegister.GAMEOVER_TIE, or other negative number if
	 *               error occured.
	 */
	public void processGameOver(int winner) {
		if (isLocal && winner >= 0) {
			log(formatString(resources.getString("youlose"),
						myModel.getPlayerName(winner)),
					LOG_MOVE);
		} else {
			super.processGameOver(winner);
		}
		myModel.disconnect(); // FIXME allow playing again
	}

	/**
	 * In local games where multiple players use the same display, locks the
	 * display for exclusive use by this player. Will block if another player
	 * has locked the display.
	 */
	protected synchronized static void lockDisplay() {
		while (locked) {
			try {
				SungkaPlayerServant.class.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
		locked = true;
	}

	/**
	 * In local games where multiple players use the same display, unlocks the
	 * display, allowing other players to use the display.
	 */
	protected synchronized static void unlockDisplay() {
		locked = false;
		SungkaPlayerServant.class.notify();
	}

	/** Runs the Text only game from the command line.  */
	public static void main(String args[]) {
		String host = null;
		int port = 0;
		for (int i = 0; i < args.length - 1; i++) {
			if (args[i].equals("-server")) {
				host = args[i + 1];
			} else if (args[i].equals("-port")) {
				try {
					port = Integer.parseInt(args[i + 1]);
				} catch (NumberFormatException e) {
				}
			}
		}

		try {
			ORB orb = ORB.init(args, null);
			SungkaPlayerServant playerServant = new SungkaPlayerServant();
			((org.omg.CORBA_2_3.ORB)orb).set_delegate(playerServant);

			SungkaPlayerView myView = new TextSungkaPlayerView(playerServant, false);

			// resolve the Object Reference in Naming
			SungkaRegister sungkaServant = null;
			
			while (sungkaServant == null) {
				while (host == null && port == 0) {
					System.out.print("Enter server address (blank for " +
							DEFAULT_HOST + ":" + DEFAULT_PORT + "): ");
					try {
						host = input.readLine().trim();
					} catch (IOException e) {
						e.printStackTrace();
						System.exit(1);
					}
					int findColon = host.lastIndexOf(':'); 
					if (findColon >= 0) {
						try {
							port = Integer.parseInt(host.substring(findColon + 1));
						} catch (NumberFormatException e) {
							host = null;
							System.out.println("Bad port number!");
							continue;
						}
						host = host.substring(0, findColon - 1);
						host = host.trim();
					}
				}
		
				if (host.equals("")) {
					host = DEFAULT_HOST;
				}
				if (port == 0) {
					port = DEFAULT_PORT;
				}
				sungkaServant = playerServant.connectToSungka(host, port);
				if (sungkaServant == null) {
					host = null;
					port = 0;
				}
			}

			// Input user name
			String playerName = "";
			System.out.print("Enter your name (blank for observer): ");
			try {
				playerName = input.readLine();
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
			}
			
			// If user name was blank, we are an observer
			boolean observer = (playerName == null || playerName.equals(""));
			boolean playerJoined = false;
			playerJoined = playerServant.joinSungkaGame(sungkaServant,
							(observer ? "Observer " + new Random().nextInt(100)
							 : playerName), observer);
			// No need for reference to registration server any more.
			sungkaServant._release();

			// Start the event loop
			if (playerJoined) {
				if (!observer) {
					// For now, we say we are ready immediately. FIXME
					playerServant.myBoard.readyToPlay();
				}		
				playerServant.mainLoop();
			}
			playerServant.shutdown();
		} catch (Exception e) {
			System.out.println("ERROR : " + e) ;
			e.printStackTrace(System.out);
			System.exit(1);
		}
	}
}
